# WARNING: Do not edit by hand, this file was generated by Crank:
#
#   https://github.com/gocardless/crank
#

import json

import requests
import responses
from nose.tools import (
  assert_equal,
  assert_is_instance,
  assert_is_none,
  assert_is_not_none,
  assert_not_equal,
  assert_raises
)

from gocardless_pro.errors import MalformedResponseError
from gocardless_pro import resources
from gocardless_pro import list_response

from .. import helpers
  

@responses.activate
def test_webhooks_list():
    fixture = helpers.load_fixture('webhooks')['list']
    helpers.stub_response(fixture)
    response = helpers.client.webhooks.list(*fixture['url_params'])
    body = fixture['body']['webhooks']

    assert_is_instance(response, list_response.ListResponse)
    assert_is_instance(response.records[0], resources.Webhook)

    assert_equal(response.before, fixture['body']['meta']['cursors']['before'])
    assert_equal(response.after, fixture['body']['meta']['cursors']['after'])
    assert_is_none(responses.calls[-1].request.headers.get('Idempotency-Key'))
    assert_equal([r.created_at for r in response.records],
                 [b.get('created_at') for b in body])
    assert_equal([r.id for r in response.records],
                 [b.get('id') for b in body])
    assert_equal([r.is_test for r in response.records],
                 [b.get('is_test') for b in body])
    assert_equal([r.request_body for r in response.records],
                 [b.get('request_body') for b in body])
    assert_equal([r.request_headers for r in response.records],
                 [b.get('request_headers') for b in body])
    assert_equal([r.response_body for r in response.records],
                 [b.get('response_body') for b in body])
    assert_equal([r.response_body_truncated for r in response.records],
                 [b.get('response_body_truncated') for b in body])
    assert_equal([r.response_code for r in response.records],
                 [b.get('response_code') for b in body])
    assert_equal([r.response_headers for r in response.records],
                 [b.get('response_headers') for b in body])
    assert_equal([r.response_headers_content_truncated for r in response.records],
                 [b.get('response_headers_content_truncated') for b in body])
    assert_equal([r.response_headers_count_truncated for r in response.records],
                 [b.get('response_headers_count_truncated') for b in body])
    assert_equal([r.successful for r in response.records],
                 [b.get('successful') for b in body])
    assert_equal([r.url for r in response.records],
                 [b.get('url') for b in body])

@responses.activate
def test_timeout_webhooks_list_retries():
    fixture = helpers.load_fixture('webhooks')['list']
    with helpers.stub_timeout_then_response(fixture) as rsps:
      response = helpers.client.webhooks.list(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['webhooks']

    assert_is_instance(response, list_response.ListResponse)
    assert_is_instance(response.records[0], resources.Webhook)

    assert_equal(response.before, fixture['body']['meta']['cursors']['before'])
    assert_equal(response.after, fixture['body']['meta']['cursors']['after'])

def test_502_webhooks_list_retries():
    fixture = helpers.load_fixture('webhooks')['list']
    with helpers.stub_502_then_response(fixture) as rsps:
      response = helpers.client.webhooks.list(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['webhooks']

    assert_is_instance(response, list_response.ListResponse)
    assert_is_instance(response.records[0], resources.Webhook)

    assert_equal(response.before, fixture['body']['meta']['cursors']['before'])
    assert_equal(response.after, fixture['body']['meta']['cursors']['after'])

@responses.activate
def test_webhooks_all():
    fixture = helpers.load_fixture('webhooks')['list']

    def callback(request):
        if 'after=123' in request.url:
          fixture['body']['meta']['cursors']['after'] = None
        else:
          fixture['body']['meta']['cursors']['after'] = '123'
        return [200, {}, json.dumps(fixture['body'])]

    url = 'http://example.com' + fixture['path_template']
    responses.add_callback(fixture['method'], url, callback)

    all_records = list(helpers.client.webhooks.all())
    assert_equal(len(all_records), len(fixture['body']['webhooks']) * 2)
    for record in all_records:
      assert_is_instance(record, resources.Webhook)
    
  

@responses.activate
def test_webhooks_get():
    fixture = helpers.load_fixture('webhooks')['get']
    helpers.stub_response(fixture)
    response = helpers.client.webhooks.get(*fixture['url_params'])
    body = fixture['body']['webhooks']

    assert_is_instance(response, resources.Webhook)
    assert_is_none(responses.calls[-1].request.headers.get('Idempotency-Key'))
    assert_equal(response.created_at, body.get('created_at'))
    assert_equal(response.id, body.get('id'))
    assert_equal(response.is_test, body.get('is_test'))
    assert_equal(response.request_body, body.get('request_body'))
    assert_equal(response.request_headers, body.get('request_headers'))
    assert_equal(response.response_body, body.get('response_body'))
    assert_equal(response.response_body_truncated, body.get('response_body_truncated'))
    assert_equal(response.response_code, body.get('response_code'))
    assert_equal(response.response_headers, body.get('response_headers'))
    assert_equal(response.response_headers_content_truncated, body.get('response_headers_content_truncated'))
    assert_equal(response.response_headers_count_truncated, body.get('response_headers_count_truncated'))
    assert_equal(response.successful, body.get('successful'))
    assert_equal(response.url, body.get('url'))

@responses.activate
def test_timeout_webhooks_get_retries():
    fixture = helpers.load_fixture('webhooks')['get']
    with helpers.stub_timeout_then_response(fixture) as rsps:
      response = helpers.client.webhooks.get(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['webhooks']

    assert_is_instance(response, resources.Webhook)

def test_502_webhooks_get_retries():
    fixture = helpers.load_fixture('webhooks')['get']
    with helpers.stub_502_then_response(fixture) as rsps:
      response = helpers.client.webhooks.get(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['webhooks']

    assert_is_instance(response, resources.Webhook)
  

@responses.activate
def test_webhooks_retry():
    fixture = helpers.load_fixture('webhooks')['retry']
    helpers.stub_response(fixture)
    response = helpers.client.webhooks.retry(*fixture['url_params'])
    body = fixture['body']['webhooks']

    assert_is_instance(response, resources.Webhook)
    assert_is_not_none(responses.calls[-1].request.headers.get('Idempotency-Key'))
    assert_equal(response.created_at, body.get('created_at'))
    assert_equal(response.id, body.get('id'))
    assert_equal(response.is_test, body.get('is_test'))
    assert_equal(response.request_body, body.get('request_body'))
    assert_equal(response.request_headers, body.get('request_headers'))
    assert_equal(response.response_body, body.get('response_body'))
    assert_equal(response.response_body_truncated, body.get('response_body_truncated'))
    assert_equal(response.response_code, body.get('response_code'))
    assert_equal(response.response_headers, body.get('response_headers'))
    assert_equal(response.response_headers_content_truncated, body.get('response_headers_content_truncated'))
    assert_equal(response.response_headers_count_truncated, body.get('response_headers_count_truncated'))
    assert_equal(response.successful, body.get('successful'))
    assert_equal(response.url, body.get('url'))

def test_timeout_webhooks_retry_doesnt_retry():
    fixture = helpers.load_fixture('webhooks')['retry']
    with helpers.stub_timeout(fixture) as rsps:
      with assert_raises(requests.ConnectTimeout):
        response = helpers.client.webhooks.retry(*fixture['url_params'])
      assert_equal(1, len(rsps.calls))

def test_502_webhooks_retry_doesnt_retry():
    fixture = helpers.load_fixture('webhooks')['retry']
    with helpers.stub_502(fixture) as rsps:
      with assert_raises(MalformedResponseError):
        response = helpers.client.webhooks.retry(*fixture['url_params'])
      assert_equal(1, len(rsps.calls))
  



# WARNING: Do not edit by hand, this file was generated by Crank:
#
#   https://github.com/gocardless/crank
#

from . import base_service
from .. import resources
from ..paginator import Paginator
from .. import errors

class InstalmentSchedulesService(base_service.BaseService):
    """Service class that provides access to the instalment_schedules
    endpoints of the GoCardless Pro API.
    """

    RESOURCE_CLASS = resources.InstalmentSchedule
    RESOURCE_NAME = 'instalment_schedules'


    def create_with_dates(self,params=None, headers=None):
        """Create (with dates).

        Creates a new instalment schedule object, along with the associated
        payments. This
        API is recommended if you know the specific dates you wish to charge.
        Otherwise,
        please check out the [scheduling
        version](#instalment-schedules-create-with-schedule).
        
        The `instalments` property is an array of payment properties (`amount`
        and
        `charge_date`).
        
        It can take quite a while to create the associated payments, so the API
        will return
        the status as `pending` initially. When processing has completed, a
        subsequent GET
        request for the instalment schedule will either have the status
        `success` and link
        to the created payments, or the status `error` and detailed information
        about the
        failures.

        Args:
              params (dict, optional): Request body.

        Returns:
              InstalmentSchedule
        """
        path = '/instalment_schedules'
        
        if params is not None:
            params = {self._envelope_key(): params}

        try:
          response = self._perform_request('POST', path, params, headers,
                                            retry_failures=True)
        except errors.IdempotentCreationConflictError as err:
          if self.raise_on_idempotency_conflict:
            raise err
          return self.get(identity=err.conflicting_resource_id,
                          params=params,
                          headers=headers)
        return self._resource_for(response)
  

    def create_with_schedule(self,params=None, headers=None):
        """Create (with schedule).

        Creates a new instalment schedule object, along with the associated
        payments. This
        API is recommended if you wish to use the GoCardless scheduling logic.
        For finer
        control over the individual dates, please check out the [alternative
        version](#instalment-schedules-create-with-dates).
        
        It can take quite a while to create the associated payments, so the API
        will return
        the status as `pending` initially. When processing has completed, a
        subsequent
        GET request for the instalment schedule will either have the status
        `success` and link to
        the created payments, or the status `error` and detailed information
        about the
        failures.

        Args:
              params (dict, optional): Request body.

        Returns:
              InstalmentSchedule
        """
        path = '/instalment_schedules'
        
        if params is not None:
            params = {self._envelope_key(): params}

        try:
          response = self._perform_request('POST', path, params, headers,
                                            retry_failures=True)
        except errors.IdempotentCreationConflictError as err:
          if self.raise_on_idempotency_conflict:
            raise err
          return self.get(identity=err.conflicting_resource_id,
                          params=params,
                          headers=headers)
        return self._resource_for(response)
  

    def list(self,params=None, headers=None):
        """List instalment schedules.

        Returns a [cursor-paginated](#api-usage-cursor-pagination) list of your
        instalment schedules.

        Args:
              params (dict, optional): Query string parameters.

        Returns:
              ListResponse of InstalmentSchedule instances
        """
        path = '/instalment_schedules'
        

        response = self._perform_request('GET', path, params, headers,
                                         retry_failures=True)
        return self._resource_for(response)

    def all(self, params=None):
        if params is None:
            params = {}
        return Paginator(self, params)
    
  

    def get(self,identity,params=None, headers=None):
        """Get a single instalment schedule.

        Retrieves the details of an existing instalment schedule.

        Args:
              identity (string): Unique identifier, beginning with "IS".
              params (dict, optional): Query string parameters.

        Returns:
              InstalmentSchedule
        """
        path = self._sub_url_params('/instalment_schedules/:identity', {
          
            'identity': identity,
          })
        

        response = self._perform_request('GET', path, params, headers,
                                         retry_failures=True)
        return self._resource_for(response)
  

    def update(self,identity,params=None, headers=None):
        """Update an instalment schedule.

        Updates an instalment schedule. This accepts only the metadata
        parameter.

        Args:
              identity (string): Unique identifier, beginning with "IS".
              params (dict, optional): Request body.

        Returns:
              InstalmentSchedule
        """
        path = self._sub_url_params('/instalment_schedules/:identity', {
          
            'identity': identity,
          })
        
        if params is not None:
            params = {self._envelope_key(): params}

        response = self._perform_request('PUT', path, params, headers,
                                         retry_failures=True)
        return self._resource_for(response)
  

    def cancel(self,identity,params=None, headers=None):
        """Cancel an instalment schedule.

        Immediately cancels an instalment schedule; no further payments will be
        collected for it.
        
        This will fail with a `cancellation_failed` error if the instalment
        schedule is already cancelled or has completed.

        Args:
              identity (string): Unique identifier, beginning with "IS".
              params (dict, optional): Request body.

        Returns:
              InstalmentSchedule
        """
        path = self._sub_url_params('/instalment_schedules/:identity/actions/cancel', {
          
            'identity': identity,
          })
        
        if params is not None:
            params = {'data': params}
        response = self._perform_request('POST', path, params, headers,
                                         retry_failures=False)
        return self._resource_for(response)
  



# WARNING: Do not edit by hand, this file was generated by Crank:
#
#   https://github.com/gocardless/crank
#

from .bank_authorisations_service import BankAuthorisationsService
from .bank_details_lookups_service import BankDetailsLookupsService
from .billing_requests_service import BillingRequestsService
from .billing_request_flows_service import BillingRequestFlowsService
from .billing_request_templates_service import BillingRequestTemplatesService
from .blocks_service import BlocksService
from .creditors_service import CreditorsService
from .creditor_bank_accounts_service import CreditorBankAccountsService
from .currency_exchange_rates_service import CurrencyExchangeRatesService
from .customers_service import CustomersService
from .customer_bank_accounts_service import CustomerBankAccountsService
from .customer_notifications_service import CustomerNotificationsService
from .events_service import EventsService
from .instalment_schedules_service import InstalmentSchedulesService
from .institutions_service import InstitutionsService
from .mandates_service import MandatesService
from .mandate_imports_service import MandateImportsService
from .mandate_import_entries_service import MandateImportEntriesService
from .mandate_pdfs_service import MandatePdfsService
from .payer_authorisations_service import PayerAuthorisationsService
from .payments_service import PaymentsService
from .payouts_service import PayoutsService
from .payout_items_service import PayoutItemsService
from .redirect_flows_service import RedirectFlowsService
from .refunds_service import RefundsService
from .scenario_simulators_service import ScenarioSimulatorsService
from .subscriptions_service import SubscriptionsService
from .tax_rates_service import TaxRatesService
from .webhooks_service import WebhooksService



# WARNING: Do not edit by hand, this file was generated by Crank:
#
#   https://github.com/gocardless/crank
#

import json

import requests
import responses
from nose.tools import (
  assert_equal,
  assert_is_instance,
  assert_is_none,
  assert_is_not_none,
  assert_not_equal,
  assert_raises
)

from gocardless_pro.errors import MalformedResponseError
from gocardless_pro import resources
from gocardless_pro import list_response

from .. import helpers
  

@responses.activate
def test_institutions_list():
    fixture = helpers.load_fixture('institutions')['list']
    helpers.stub_response(fixture)
    response = helpers.client.institutions.list(*fixture['url_params'])
    body = fixture['body']['institutions']

    assert_is_instance(response, list_response.ListResponse)
    assert_is_instance(response.records[0], resources.Institution)

    assert_equal(response.before, fixture['body']['meta']['cursors']['before'])
    assert_equal(response.after, fixture['body']['meta']['cursors']['after'])
    assert_is_none(responses.calls[-1].request.headers.get('Idempotency-Key'))
    assert_equal([r.country_code for r in response.records],
                 [b.get('country_code') for b in body])
    assert_equal([r.icon_url for r in response.records],
                 [b.get('icon_url') for b in body])
    assert_equal([r.id for r in response.records],
                 [b.get('id') for b in body])
    assert_equal([r.logo_url for r in response.records],
                 [b.get('logo_url') for b in body])
    assert_equal([r.name for r in response.records],
                 [b.get('name') for b in body])

@responses.activate
def test_timeout_institutions_list_retries():
    fixture = helpers.load_fixture('institutions')['list']
    with helpers.stub_timeout_then_response(fixture) as rsps:
      response = helpers.client.institutions.list(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['institutions']

    assert_is_instance(response, list_response.ListResponse)
    assert_is_instance(response.records[0], resources.Institution)

    assert_equal(response.before, fixture['body']['meta']['cursors']['before'])
    assert_equal(response.after, fixture['body']['meta']['cursors']['after'])

def test_502_institutions_list_retries():
    fixture = helpers.load_fixture('institutions')['list']
    with helpers.stub_502_then_response(fixture) as rsps:
      response = helpers.client.institutions.list(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['institutions']

    assert_is_instance(response, list_response.ListResponse)
    assert_is_instance(response.records[0], resources.Institution)

    assert_equal(response.before, fixture['body']['meta']['cursors']['before'])
    assert_equal(response.after, fixture['body']['meta']['cursors']['after'])

@responses.activate
def test_institutions_all():
    fixture = helpers.load_fixture('institutions')['list']

    def callback(request):
        if 'after=123' in request.url:
          fixture['body']['meta']['cursors']['after'] = None
        else:
          fixture['body']['meta']['cursors']['after'] = '123'
        return [200, {}, json.dumps(fixture['body'])]

    url = 'http://example.com' + fixture['path_template']
    responses.add_callback(fixture['method'], url, callback)

    all_records = list(helpers.client.institutions.all())
    assert_equal(len(all_records), len(fixture['body']['institutions']) * 2)
    for record in all_records:
      assert_is_instance(record, resources.Institution)
    
  



# WARNING: Do not edit by hand, this file was generated by Crank:
#
#   https://github.com/gocardless/crank
#

class BankAuthorisation(object):
    """A thin wrapper around a bank_authorisation, providing easy access to its
    attributes.

    Example:
      bank_authorisation = client.bank_authorisations.get()
      bank_authorisation.id
    """

    def __init__(self, attributes, api_response):
        self.attributes = attributes
        self.api_response = api_response

    @property
    def authorisation_type(self):
        return self.attributes.get('authorisation_type')
  

    @property
    def authorised_at(self):
        return self.attributes.get('authorised_at')
  

    @property
    def created_at(self):
        return self.attributes.get('created_at')
  

    @property
    def expires_at(self):
        return self.attributes.get('expires_at')
  

    @property
    def id(self):
        return self.attributes.get('id')
  

    @property
    def last_visited_at(self):
        return self.attributes.get('last_visited_at')
  

    @property
    def links(self):
        return self.Links(self.attributes.get('links'))
  

    @property
    def redirect_uri(self):
        return self.attributes.get('redirect_uri')
  

    @property
    def url(self):
        return self.attributes.get('url')
  


  

  

  

  

  

  

  
    class Links(object):
        """Wrapper for the response's 'links' attribute."""

        def __init__(self, attributes):
            self.attributes = attributes
    
        @property
        def billing_request(self):
            return self.attributes.get('billing_request')
    
        @property
        def institution(self):
            return self.attributes.get('institution')
    
  

  

  




# WARNING: Do not edit by hand, this file was generated by Crank:
#
#   https://github.com/gocardless/crank
#

import json

import requests
import responses
from nose.tools import (
  assert_equal,
  assert_is_instance,
  assert_is_none,
  assert_is_not_none,
  assert_not_equal,
  assert_raises
)

from gocardless_pro.errors import MalformedResponseError
from gocardless_pro import resources
from gocardless_pro import list_response

from .. import helpers
  

@responses.activate
def test_payer_authorisations_get():
    fixture = helpers.load_fixture('payer_authorisations')['get']
    helpers.stub_response(fixture)
    response = helpers.client.payer_authorisations.get(*fixture['url_params'])
    body = fixture['body']['payer_authorisations']

    assert_is_instance(response, resources.PayerAuthorisation)
    assert_is_none(responses.calls[-1].request.headers.get('Idempotency-Key'))
    assert_equal(response.created_at, body.get('created_at'))
    assert_equal(response.id, body.get('id'))
    assert_equal(response.incomplete_fields, body.get('incomplete_fields'))
    assert_equal(response.status, body.get('status'))
    assert_equal(response.bank_account.account_holder_name,
                 body.get('bank_account')['account_holder_name'])
    assert_equal(response.bank_account.account_number,
                 body.get('bank_account')['account_number'])
    assert_equal(response.bank_account.account_number_ending,
                 body.get('bank_account')['account_number_ending'])
    assert_equal(response.bank_account.account_number_suffix,
                 body.get('bank_account')['account_number_suffix'])
    assert_equal(response.bank_account.account_type,
                 body.get('bank_account')['account_type'])
    assert_equal(response.bank_account.bank_code,
                 body.get('bank_account')['bank_code'])
    assert_equal(response.bank_account.branch_code,
                 body.get('bank_account')['branch_code'])
    assert_equal(response.bank_account.country_code,
                 body.get('bank_account')['country_code'])
    assert_equal(response.bank_account.currency,
                 body.get('bank_account')['currency'])
    assert_equal(response.bank_account.iban,
                 body.get('bank_account')['iban'])
    assert_equal(response.bank_account.metadata,
                 body.get('bank_account')['metadata'])
    assert_equal(response.customer.address_line1,
                 body.get('customer')['address_line1'])
    assert_equal(response.customer.address_line2,
                 body.get('customer')['address_line2'])
    assert_equal(response.customer.address_line3,
                 body.get('customer')['address_line3'])
    assert_equal(response.customer.city,
                 body.get('customer')['city'])
    assert_equal(response.customer.company_name,
                 body.get('customer')['company_name'])
    assert_equal(response.customer.country_code,
                 body.get('customer')['country_code'])
    assert_equal(response.customer.danish_identity_number,
                 body.get('customer')['danish_identity_number'])
    assert_equal(response.customer.email,
                 body.get('customer')['email'])
    assert_equal(response.customer.family_name,
                 body.get('customer')['family_name'])
    assert_equal(response.customer.given_name,
                 body.get('customer')['given_name'])
    assert_equal(response.customer.locale,
                 body.get('customer')['locale'])
    assert_equal(response.customer.metadata,
                 body.get('customer')['metadata'])
    assert_equal(response.customer.postal_code,
                 body.get('customer')['postal_code'])
    assert_equal(response.customer.region,
                 body.get('customer')['region'])
    assert_equal(response.customer.swedish_identity_number,
                 body.get('customer')['swedish_identity_number'])
    assert_equal(response.links.bank_account,
                 body.get('links')['bank_account'])
    assert_equal(response.links.customer,
                 body.get('links')['customer'])
    assert_equal(response.links.mandate,
                 body.get('links')['mandate'])
    assert_equal(response.mandate.metadata,
                 body.get('mandate')['metadata'])
    assert_equal(response.mandate.payer_ip_address,
                 body.get('mandate')['payer_ip_address'])
    assert_equal(response.mandate.reference,
                 body.get('mandate')['reference'])
    assert_equal(response.mandate.scheme,
                 body.get('mandate')['scheme'])

@responses.activate
def test_timeout_payer_authorisations_get_retries():
    fixture = helpers.load_fixture('payer_authorisations')['get']
    with helpers.stub_timeout_then_response(fixture) as rsps:
      response = helpers.client.payer_authorisations.get(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['payer_authorisations']

    assert_is_instance(response, resources.PayerAuthorisation)

def test_502_payer_authorisations_get_retries():
    fixture = helpers.load_fixture('payer_authorisations')['get']
    with helpers.stub_502_then_response(fixture) as rsps:
      response = helpers.client.payer_authorisations.get(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['payer_authorisations']

    assert_is_instance(response, resources.PayerAuthorisation)
  

@responses.activate
def test_payer_authorisations_create():
    fixture = helpers.load_fixture('payer_authorisations')['create']
    helpers.stub_response(fixture)
    response = helpers.client.payer_authorisations.create(*fixture['url_params'])
    body = fixture['body']['payer_authorisations']

    assert_is_instance(response, resources.PayerAuthorisation)
    assert_is_not_none(responses.calls[-1].request.headers.get('Idempotency-Key'))
    assert_equal(response.created_at, body.get('created_at'))
    assert_equal(response.id, body.get('id'))
    assert_equal(response.incomplete_fields, body.get('incomplete_fields'))
    assert_equal(response.status, body.get('status'))
    assert_equal(response.bank_account.account_holder_name,
                 body.get('bank_account')['account_holder_name'])
    assert_equal(response.bank_account.account_number,
                 body.get('bank_account')['account_number'])
    assert_equal(response.bank_account.account_number_ending,
                 body.get('bank_account')['account_number_ending'])
    assert_equal(response.bank_account.account_number_suffix,
                 body.get('bank_account')['account_number_suffix'])
    assert_equal(response.bank_account.account_type,
                 body.get('bank_account')['account_type'])
    assert_equal(response.bank_account.bank_code,
                 body.get('bank_account')['bank_code'])
    assert_equal(response.bank_account.branch_code,
                 body.get('bank_account')['branch_code'])
    assert_equal(response.bank_account.country_code,
                 body.get('bank_account')['country_code'])
    assert_equal(response.bank_account.currency,
                 body.get('bank_account')['currency'])
    assert_equal(response.bank_account.iban,
                 body.get('bank_account')['iban'])
    assert_equal(response.bank_account.metadata,
                 body.get('bank_account')['metadata'])
    assert_equal(response.customer.address_line1,
                 body.get('customer')['address_line1'])
    assert_equal(response.customer.address_line2,
                 body.get('customer')['address_line2'])
    assert_equal(response.customer.address_line3,
                 body.get('customer')['address_line3'])
    assert_equal(response.customer.city,
                 body.get('customer')['city'])
    assert_equal(response.customer.company_name,
                 body.get('customer')['company_name'])
    assert_equal(response.customer.country_code,
                 body.get('customer')['country_code'])
    assert_equal(response.customer.danish_identity_number,
                 body.get('customer')['danish_identity_number'])
    assert_equal(response.customer.email,
                 body.get('customer')['email'])
    assert_equal(response.customer.family_name,
                 body.get('customer')['family_name'])
    assert_equal(response.customer.given_name,
                 body.get('customer')['given_name'])
    assert_equal(response.customer.locale,
                 body.get('customer')['locale'])
    assert_equal(response.customer.metadata,
                 body.get('customer')['metadata'])
    assert_equal(response.customer.postal_code,
                 body.get('customer')['postal_code'])
    assert_equal(response.customer.region,
                 body.get('customer')['region'])
    assert_equal(response.customer.swedish_identity_number,
                 body.get('customer')['swedish_identity_number'])
    assert_equal(response.links.bank_account,
                 body.get('links')['bank_account'])
    assert_equal(response.links.customer,
                 body.get('links')['customer'])
    assert_equal(response.links.mandate,
                 body.get('links')['mandate'])
    assert_equal(response.mandate.metadata,
                 body.get('mandate')['metadata'])
    assert_equal(response.mandate.payer_ip_address,
                 body.get('mandate')['payer_ip_address'])
    assert_equal(response.mandate.reference,
                 body.get('mandate')['reference'])
    assert_equal(response.mandate.scheme,
                 body.get('mandate')['scheme'])

@responses.activate
def test_payer_authorisations_create_new_idempotency_key_for_each_call():
    fixture = helpers.load_fixture('payer_authorisations')['create']
    helpers.stub_response(fixture)
    helpers.client.payer_authorisations.create(*fixture['url_params'])
    helpers.client.payer_authorisations.create(*fixture['url_params'])
    assert_not_equal(responses.calls[0].request.headers.get('Idempotency-Key'),
                     responses.calls[1].request.headers.get('Idempotency-Key'))

def test_timeout_payer_authorisations_create_idempotency_conflict():
    create_fixture = helpers.load_fixture('payer_authorisations')['create']
    get_fixture = helpers.load_fixture('payer_authorisations')['get']
    with helpers.stub_timeout_then_idempotency_conflict(create_fixture, get_fixture) as rsps:
      response = helpers.client.payer_authorisations.create(*create_fixture['url_params'])
      assert_equal(2, len(rsps.calls))

    assert_is_instance(response, resources.PayerAuthorisation)

@responses.activate
def test_timeout_payer_authorisations_create_retries():
    fixture = helpers.load_fixture('payer_authorisations')['create']
    with helpers.stub_timeout_then_response(fixture) as rsps:
      response = helpers.client.payer_authorisations.create(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['payer_authorisations']

    assert_is_instance(response, resources.PayerAuthorisation)

def test_502_payer_authorisations_create_retries():
    fixture = helpers.load_fixture('payer_authorisations')['create']
    with helpers.stub_502_then_response(fixture) as rsps:
      response = helpers.client.payer_authorisations.create(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['payer_authorisations']

    assert_is_instance(response, resources.PayerAuthorisation)
  

@responses.activate
def test_payer_authorisations_update():
    fixture = helpers.load_fixture('payer_authorisations')['update']
    helpers.stub_response(fixture)
    response = helpers.client.payer_authorisations.update(*fixture['url_params'])
    body = fixture['body']['payer_authorisations']

    assert_is_instance(response, resources.PayerAuthorisation)
    assert_is_none(responses.calls[-1].request.headers.get('Idempotency-Key'))
    assert_equal(response.created_at, body.get('created_at'))
    assert_equal(response.id, body.get('id'))
    assert_equal(response.incomplete_fields, body.get('incomplete_fields'))
    assert_equal(response.status, body.get('status'))
    assert_equal(response.bank_account.account_holder_name,
                 body.get('bank_account')['account_holder_name'])
    assert_equal(response.bank_account.account_number,
                 body.get('bank_account')['account_number'])
    assert_equal(response.bank_account.account_number_ending,
                 body.get('bank_account')['account_number_ending'])
    assert_equal(response.bank_account.account_number_suffix,
                 body.get('bank_account')['account_number_suffix'])
    assert_equal(response.bank_account.account_type,
                 body.get('bank_account')['account_type'])
    assert_equal(response.bank_account.bank_code,
                 body.get('bank_account')['bank_code'])
    assert_equal(response.bank_account.branch_code,
                 body.get('bank_account')['branch_code'])
    assert_equal(response.bank_account.country_code,
                 body.get('bank_account')['country_code'])
    assert_equal(response.bank_account.currency,
                 body.get('bank_account')['currency'])
    assert_equal(response.bank_account.iban,
                 body.get('bank_account')['iban'])
    assert_equal(response.bank_account.metadata,
                 body.get('bank_account')['metadata'])
    assert_equal(response.customer.address_line1,
                 body.get('customer')['address_line1'])
    assert_equal(response.customer.address_line2,
                 body.get('customer')['address_line2'])
    assert_equal(response.customer.address_line3,
                 body.get('customer')['address_line3'])
    assert_equal(response.customer.city,
                 body.get('customer')['city'])
    assert_equal(response.customer.company_name,
                 body.get('customer')['company_name'])
    assert_equal(response.customer.country_code,
                 body.get('customer')['country_code'])
    assert_equal(response.customer.danish_identity_number,
                 body.get('customer')['danish_identity_number'])
    assert_equal(response.customer.email,
                 body.get('customer')['email'])
    assert_equal(response.customer.family_name,
                 body.get('customer')['family_name'])
    assert_equal(response.customer.given_name,
                 body.get('customer')['given_name'])
    assert_equal(response.customer.locale,
                 body.get('customer')['locale'])
    assert_equal(response.customer.metadata,
                 body.get('customer')['metadata'])
    assert_equal(response.customer.postal_code,
                 body.get('customer')['postal_code'])
    assert_equal(response.customer.region,
                 body.get('customer')['region'])
    assert_equal(response.customer.swedish_identity_number,
                 body.get('customer')['swedish_identity_number'])
    assert_equal(response.links.bank_account,
                 body.get('links')['bank_account'])
    assert_equal(response.links.customer,
                 body.get('links')['customer'])
    assert_equal(response.links.mandate,
                 body.get('links')['mandate'])
    assert_equal(response.mandate.metadata,
                 body.get('mandate')['metadata'])
    assert_equal(response.mandate.payer_ip_address,
                 body.get('mandate')['payer_ip_address'])
    assert_equal(response.mandate.reference,
                 body.get('mandate')['reference'])
    assert_equal(response.mandate.scheme,
                 body.get('mandate')['scheme'])

@responses.activate
def test_timeout_payer_authorisations_update_retries():
    fixture = helpers.load_fixture('payer_authorisations')['update']
    with helpers.stub_timeout_then_response(fixture) as rsps:
      response = helpers.client.payer_authorisations.update(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['payer_authorisations']

    assert_is_instance(response, resources.PayerAuthorisation)

def test_502_payer_authorisations_update_retries():
    fixture = helpers.load_fixture('payer_authorisations')['update']
    with helpers.stub_502_then_response(fixture) as rsps:
      response = helpers.client.payer_authorisations.update(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['payer_authorisations']

    assert_is_instance(response, resources.PayerAuthorisation)
  

@responses.activate
def test_payer_authorisations_submit():
    fixture = helpers.load_fixture('payer_authorisations')['submit']
    helpers.stub_response(fixture)
    response = helpers.client.payer_authorisations.submit(*fixture['url_params'])
    body = fixture['body']['payer_authorisations']

    assert_is_instance(response, resources.PayerAuthorisation)
    assert_is_not_none(responses.calls[-1].request.headers.get('Idempotency-Key'))
    assert_equal(response.created_at, body.get('created_at'))
    assert_equal(response.id, body.get('id'))
    assert_equal(response.incomplete_fields, body.get('incomplete_fields'))
    assert_equal(response.status, body.get('status'))
    assert_equal(response.bank_account.account_holder_name,
                 body.get('bank_account')['account_holder_name'])
    assert_equal(response.bank_account.account_number,
                 body.get('bank_account')['account_number'])
    assert_equal(response.bank_account.account_number_ending,
                 body.get('bank_account')['account_number_ending'])
    assert_equal(response.bank_account.account_number_suffix,
                 body.get('bank_account')['account_number_suffix'])
    assert_equal(response.bank_account.account_type,
                 body.get('bank_account')['account_type'])
    assert_equal(response.bank_account.bank_code,
                 body.get('bank_account')['bank_code'])
    assert_equal(response.bank_account.branch_code,
                 body.get('bank_account')['branch_code'])
    assert_equal(response.bank_account.country_code,
                 body.get('bank_account')['country_code'])
    assert_equal(response.bank_account.currency,
                 body.get('bank_account')['currency'])
    assert_equal(response.bank_account.iban,
                 body.get('bank_account')['iban'])
    assert_equal(response.bank_account.metadata,
                 body.get('bank_account')['metadata'])
    assert_equal(response.customer.address_line1,
                 body.get('customer')['address_line1'])
    assert_equal(response.customer.address_line2,
                 body.get('customer')['address_line2'])
    assert_equal(response.customer.address_line3,
                 body.get('customer')['address_line3'])
    assert_equal(response.customer.city,
                 body.get('customer')['city'])
    assert_equal(response.customer.company_name,
                 body.get('customer')['company_name'])
    assert_equal(response.customer.country_code,
                 body.get('customer')['country_code'])
    assert_equal(response.customer.danish_identity_number,
                 body.get('customer')['danish_identity_number'])
    assert_equal(response.customer.email,
                 body.get('customer')['email'])
    assert_equal(response.customer.family_name,
                 body.get('customer')['family_name'])
    assert_equal(response.customer.given_name,
                 body.get('customer')['given_name'])
    assert_equal(response.customer.locale,
                 body.get('customer')['locale'])
    assert_equal(response.customer.metadata,
                 body.get('customer')['metadata'])
    assert_equal(response.customer.postal_code,
                 body.get('customer')['postal_code'])
    assert_equal(response.customer.region,
                 body.get('customer')['region'])
    assert_equal(response.customer.swedish_identity_number,
                 body.get('customer')['swedish_identity_number'])
    assert_equal(response.links.bank_account,
                 body.get('links')['bank_account'])
    assert_equal(response.links.customer,
                 body.get('links')['customer'])
    assert_equal(response.links.mandate,
                 body.get('links')['mandate'])
    assert_equal(response.mandate.metadata,
                 body.get('mandate')['metadata'])
    assert_equal(response.mandate.payer_ip_address,
                 body.get('mandate')['payer_ip_address'])
    assert_equal(response.mandate.reference,
                 body.get('mandate')['reference'])
    assert_equal(response.mandate.scheme,
                 body.get('mandate')['scheme'])

def test_timeout_payer_authorisations_submit_doesnt_retry():
    fixture = helpers.load_fixture('payer_authorisations')['submit']
    with helpers.stub_timeout(fixture) as rsps:
      with assert_raises(requests.ConnectTimeout):
        response = helpers.client.payer_authorisations.submit(*fixture['url_params'])
      assert_equal(1, len(rsps.calls))

def test_502_payer_authorisations_submit_doesnt_retry():
    fixture = helpers.load_fixture('payer_authorisations')['submit']
    with helpers.stub_502(fixture) as rsps:
      with assert_raises(MalformedResponseError):
        response = helpers.client.payer_authorisations.submit(*fixture['url_params'])
      assert_equal(1, len(rsps.calls))
  

@responses.activate
def test_payer_authorisations_confirm():
    fixture = helpers.load_fixture('payer_authorisations')['confirm']
    helpers.stub_response(fixture)
    response = helpers.client.payer_authorisations.confirm(*fixture['url_params'])
    body = fixture['body']['payer_authorisations']

    assert_is_instance(response, resources.PayerAuthorisation)
    assert_is_not_none(responses.calls[-1].request.headers.get('Idempotency-Key'))
    assert_equal(response.created_at, body.get('created_at'))
    assert_equal(response.id, body.get('id'))
    assert_equal(response.incomplete_fields, body.get('incomplete_fields'))
    assert_equal(response.status, body.get('status'))
    assert_equal(response.bank_account.account_holder_name,
                 body.get('bank_account')['account_holder_name'])
    assert_equal(response.bank_account.account_number,
                 body.get('bank_account')['account_number'])
    assert_equal(response.bank_account.account_number_ending,
                 body.get('bank_account')['account_number_ending'])
    assert_equal(response.bank_account.account_number_suffix,
                 body.get('bank_account')['account_number_suffix'])
    assert_equal(response.bank_account.account_type,
                 body.get('bank_account')['account_type'])
    assert_equal(response.bank_account.bank_code,
                 body.get('bank_account')['bank_code'])
    assert_equal(response.bank_account.branch_code,
                 body.get('bank_account')['branch_code'])
    assert_equal(response.bank_account.country_code,
                 body.get('bank_account')['country_code'])
    assert_equal(response.bank_account.currency,
                 body.get('bank_account')['currency'])
    assert_equal(response.bank_account.iban,
                 body.get('bank_account')['iban'])
    assert_equal(response.bank_account.metadata,
                 body.get('bank_account')['metadata'])
    assert_equal(response.customer.address_line1,
                 body.get('customer')['address_line1'])
    assert_equal(response.customer.address_line2,
                 body.get('customer')['address_line2'])
    assert_equal(response.customer.address_line3,
                 body.get('customer')['address_line3'])
    assert_equal(response.customer.city,
                 body.get('customer')['city'])
    assert_equal(response.customer.company_name,
                 body.get('customer')['company_name'])
    assert_equal(response.customer.country_code,
                 body.get('customer')['country_code'])
    assert_equal(response.customer.danish_identity_number,
                 body.get('customer')['danish_identity_number'])
    assert_equal(response.customer.email,
                 body.get('customer')['email'])
    assert_equal(response.customer.family_name,
                 body.get('customer')['family_name'])
    assert_equal(response.customer.given_name,
                 body.get('customer')['given_name'])
    assert_equal(response.customer.locale,
                 body.get('customer')['locale'])
    assert_equal(response.customer.metadata,
                 body.get('customer')['metadata'])
    assert_equal(response.customer.postal_code,
                 body.get('customer')['postal_code'])
    assert_equal(response.customer.region,
                 body.get('customer')['region'])
    assert_equal(response.customer.swedish_identity_number,
                 body.get('customer')['swedish_identity_number'])
    assert_equal(response.links.bank_account,
                 body.get('links')['bank_account'])
    assert_equal(response.links.customer,
                 body.get('links')['customer'])
    assert_equal(response.links.mandate,
                 body.get('links')['mandate'])
    assert_equal(response.mandate.metadata,
                 body.get('mandate')['metadata'])
    assert_equal(response.mandate.payer_ip_address,
                 body.get('mandate')['payer_ip_address'])
    assert_equal(response.mandate.reference,
                 body.get('mandate')['reference'])
    assert_equal(response.mandate.scheme,
                 body.get('mandate')['scheme'])

def test_timeout_payer_authorisations_confirm_doesnt_retry():
    fixture = helpers.load_fixture('payer_authorisations')['confirm']
    with helpers.stub_timeout(fixture) as rsps:
      with assert_raises(requests.ConnectTimeout):
        response = helpers.client.payer_authorisations.confirm(*fixture['url_params'])
      assert_equal(1, len(rsps.calls))

def test_502_payer_authorisations_confirm_doesnt_retry():
    fixture = helpers.load_fixture('payer_authorisations')['confirm']
    with helpers.stub_502(fixture) as rsps:
      with assert_raises(MalformedResponseError):
        response = helpers.client.payer_authorisations.confirm(*fixture['url_params'])
      assert_equal(1, len(rsps.calls))
  



# WARNING: Do not edit by hand, this file was generated by Crank:
#
#   https://github.com/gocardless/crank
#

class Webhook(object):
    """A thin wrapper around a webhook, providing easy access to its
    attributes.

    Example:
      webhook = client.webhooks.get()
      webhook.id
    """

    def __init__(self, attributes, api_response):
        self.attributes = attributes
        self.api_response = api_response

    @property
    def created_at(self):
        return self.attributes.get('created_at')
  

    @property
    def id(self):
        return self.attributes.get('id')
  

    @property
    def is_test(self):
        return self.attributes.get('is_test')
  

    @property
    def request_body(self):
        return self.attributes.get('request_body')
  

    @property
    def request_headers(self):
        return self.attributes.get('request_headers')
  

    @property
    def response_body(self):
        return self.attributes.get('response_body')
  

    @property
    def response_body_truncated(self):
        return self.attributes.get('response_body_truncated')
  

    @property
    def response_code(self):
        return self.attributes.get('response_code')
  

    @property
    def response_headers(self):
        return self.attributes.get('response_headers')
  

    @property
    def response_headers_content_truncated(self):
        return self.attributes.get('response_headers_content_truncated')
  

    @property
    def response_headers_count_truncated(self):
        return self.attributes.get('response_headers_count_truncated')
  

    @property
    def successful(self):
        return self.attributes.get('successful')
  

    @property
    def url(self):
        return self.attributes.get('url')
  


  

  

  

  

  

  

  

  

  

  

  

  

  




# WARNING: Do not edit by hand, this file was generated by Crank:
#
#   https://github.com/gocardless/crank
#

import json

import requests
import responses
from nose.tools import (
  assert_equal,
  assert_is_instance,
  assert_is_none,
  assert_is_not_none,
  assert_not_equal,
  assert_raises
)

from gocardless_pro.errors import MalformedResponseError
from gocardless_pro import resources
from gocardless_pro import list_response

from .. import helpers
  

@responses.activate
def test_subscriptions_create():
    fixture = helpers.load_fixture('subscriptions')['create']
    helpers.stub_response(fixture)
    response = helpers.client.subscriptions.create(*fixture['url_params'])
    body = fixture['body']['subscriptions']

    assert_is_instance(response, resources.Subscription)
    assert_is_not_none(responses.calls[-1].request.headers.get('Idempotency-Key'))
    assert_equal(response.amount, body.get('amount'))
    assert_equal(response.app_fee, body.get('app_fee'))
    assert_equal(response.count, body.get('count'))
    assert_equal(response.created_at, body.get('created_at'))
    assert_equal(response.currency, body.get('currency'))
    assert_equal(response.day_of_month, body.get('day_of_month'))
    assert_equal(response.earliest_charge_date_after_resume, body.get('earliest_charge_date_after_resume'))
    assert_equal(response.end_date, body.get('end_date'))
    assert_equal(response.id, body.get('id'))
    assert_equal(response.interval, body.get('interval'))
    assert_equal(response.interval_unit, body.get('interval_unit'))
    assert_equal(response.metadata, body.get('metadata'))
    assert_equal(response.month, body.get('month'))
    assert_equal(response.name, body.get('name'))
    assert_equal(response.payment_reference, body.get('payment_reference'))
    assert_equal(response.retry_if_possible, body.get('retry_if_possible'))
    assert_equal(response.start_date, body.get('start_date'))
    assert_equal(response.status, body.get('status'))
    assert_equal(response.upcoming_payments, body.get('upcoming_payments'))
    assert_equal(response.links.mandate,
                 body.get('links')['mandate'])

@responses.activate
def test_subscriptions_create_new_idempotency_key_for_each_call():
    fixture = helpers.load_fixture('subscriptions')['create']
    helpers.stub_response(fixture)
    helpers.client.subscriptions.create(*fixture['url_params'])
    helpers.client.subscriptions.create(*fixture['url_params'])
    assert_not_equal(responses.calls[0].request.headers.get('Idempotency-Key'),
                     responses.calls[1].request.headers.get('Idempotency-Key'))

def test_timeout_subscriptions_create_idempotency_conflict():
    create_fixture = helpers.load_fixture('subscriptions')['create']
    get_fixture = helpers.load_fixture('subscriptions')['get']
    with helpers.stub_timeout_then_idempotency_conflict(create_fixture, get_fixture) as rsps:
      response = helpers.client.subscriptions.create(*create_fixture['url_params'])
      assert_equal(2, len(rsps.calls))

    assert_is_instance(response, resources.Subscription)

@responses.activate
def test_timeout_subscriptions_create_retries():
    fixture = helpers.load_fixture('subscriptions')['create']
    with helpers.stub_timeout_then_response(fixture) as rsps:
      response = helpers.client.subscriptions.create(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['subscriptions']

    assert_is_instance(response, resources.Subscription)

def test_502_subscriptions_create_retries():
    fixture = helpers.load_fixture('subscriptions')['create']
    with helpers.stub_502_then_response(fixture) as rsps:
      response = helpers.client.subscriptions.create(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['subscriptions']

    assert_is_instance(response, resources.Subscription)
  

@responses.activate
def test_subscriptions_list():
    fixture = helpers.load_fixture('subscriptions')['list']
    helpers.stub_response(fixture)
    response = helpers.client.subscriptions.list(*fixture['url_params'])
    body = fixture['body']['subscriptions']

    assert_is_instance(response, list_response.ListResponse)
    assert_is_instance(response.records[0], resources.Subscription)

    assert_equal(response.before, fixture['body']['meta']['cursors']['before'])
    assert_equal(response.after, fixture['body']['meta']['cursors']['after'])
    assert_is_none(responses.calls[-1].request.headers.get('Idempotency-Key'))
    assert_equal([r.amount for r in response.records],
                 [b.get('amount') for b in body])
    assert_equal([r.app_fee for r in response.records],
                 [b.get('app_fee') for b in body])
    assert_equal([r.count for r in response.records],
                 [b.get('count') for b in body])
    assert_equal([r.created_at for r in response.records],
                 [b.get('created_at') for b in body])
    assert_equal([r.currency for r in response.records],
                 [b.get('currency') for b in body])
    assert_equal([r.day_of_month for r in response.records],
                 [b.get('day_of_month') for b in body])
    assert_equal([r.earliest_charge_date_after_resume for r in response.records],
                 [b.get('earliest_charge_date_after_resume') for b in body])
    assert_equal([r.end_date for r in response.records],
                 [b.get('end_date') for b in body])
    assert_equal([r.id for r in response.records],
                 [b.get('id') for b in body])
    assert_equal([r.interval for r in response.records],
                 [b.get('interval') for b in body])
    assert_equal([r.interval_unit for r in response.records],
                 [b.get('interval_unit') for b in body])
    assert_equal([r.metadata for r in response.records],
                 [b.get('metadata') for b in body])
    assert_equal([r.month for r in response.records],
                 [b.get('month') for b in body])
    assert_equal([r.name for r in response.records],
                 [b.get('name') for b in body])
    assert_equal([r.payment_reference for r in response.records],
                 [b.get('payment_reference') for b in body])
    assert_equal([r.retry_if_possible for r in response.records],
                 [b.get('retry_if_possible') for b in body])
    assert_equal([r.start_date for r in response.records],
                 [b.get('start_date') for b in body])
    assert_equal([r.status for r in response.records],
                 [b.get('status') for b in body])
    assert_equal([r.upcoming_payments for r in response.records],
                 [b.get('upcoming_payments') for b in body])

@responses.activate
def test_timeout_subscriptions_list_retries():
    fixture = helpers.load_fixture('subscriptions')['list']
    with helpers.stub_timeout_then_response(fixture) as rsps:
      response = helpers.client.subscriptions.list(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['subscriptions']

    assert_is_instance(response, list_response.ListResponse)
    assert_is_instance(response.records[0], resources.Subscription)

    assert_equal(response.before, fixture['body']['meta']['cursors']['before'])
    assert_equal(response.after, fixture['body']['meta']['cursors']['after'])

def test_502_subscriptions_list_retries():
    fixture = helpers.load_fixture('subscriptions')['list']
    with helpers.stub_502_then_response(fixture) as rsps:
      response = helpers.client.subscriptions.list(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['subscriptions']

    assert_is_instance(response, list_response.ListResponse)
    assert_is_instance(response.records[0], resources.Subscription)

    assert_equal(response.before, fixture['body']['meta']['cursors']['before'])
    assert_equal(response.after, fixture['body']['meta']['cursors']['after'])

@responses.activate
def test_subscriptions_all():
    fixture = helpers.load_fixture('subscriptions')['list']

    def callback(request):
        if 'after=123' in request.url:
          fixture['body']['meta']['cursors']['after'] = None
        else:
          fixture['body']['meta']['cursors']['after'] = '123'
        return [200, {}, json.dumps(fixture['body'])]

    url = 'http://example.com' + fixture['path_template']
    responses.add_callback(fixture['method'], url, callback)

    all_records = list(helpers.client.subscriptions.all())
    assert_equal(len(all_records), len(fixture['body']['subscriptions']) * 2)
    for record in all_records:
      assert_is_instance(record, resources.Subscription)
    
  

@responses.activate
def test_subscriptions_get():
    fixture = helpers.load_fixture('subscriptions')['get']
    helpers.stub_response(fixture)
    response = helpers.client.subscriptions.get(*fixture['url_params'])
    body = fixture['body']['subscriptions']

    assert_is_instance(response, resources.Subscription)
    assert_is_none(responses.calls[-1].request.headers.get('Idempotency-Key'))
    assert_equal(response.amount, body.get('amount'))
    assert_equal(response.app_fee, body.get('app_fee'))
    assert_equal(response.count, body.get('count'))
    assert_equal(response.created_at, body.get('created_at'))
    assert_equal(response.currency, body.get('currency'))
    assert_equal(response.day_of_month, body.get('day_of_month'))
    assert_equal(response.earliest_charge_date_after_resume, body.get('earliest_charge_date_after_resume'))
    assert_equal(response.end_date, body.get('end_date'))
    assert_equal(response.id, body.get('id'))
    assert_equal(response.interval, body.get('interval'))
    assert_equal(response.interval_unit, body.get('interval_unit'))
    assert_equal(response.metadata, body.get('metadata'))
    assert_equal(response.month, body.get('month'))
    assert_equal(response.name, body.get('name'))
    assert_equal(response.payment_reference, body.get('payment_reference'))
    assert_equal(response.retry_if_possible, body.get('retry_if_possible'))
    assert_equal(response.start_date, body.get('start_date'))
    assert_equal(response.status, body.get('status'))
    assert_equal(response.upcoming_payments, body.get('upcoming_payments'))
    assert_equal(response.links.mandate,
                 body.get('links')['mandate'])

@responses.activate
def test_timeout_subscriptions_get_retries():
    fixture = helpers.load_fixture('subscriptions')['get']
    with helpers.stub_timeout_then_response(fixture) as rsps:
      response = helpers.client.subscriptions.get(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['subscriptions']

    assert_is_instance(response, resources.Subscription)

def test_502_subscriptions_get_retries():
    fixture = helpers.load_fixture('subscriptions')['get']
    with helpers.stub_502_then_response(fixture) as rsps:
      response = helpers.client.subscriptions.get(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['subscriptions']

    assert_is_instance(response, resources.Subscription)
  

@responses.activate
def test_subscriptions_update():
    fixture = helpers.load_fixture('subscriptions')['update']
    helpers.stub_response(fixture)
    response = helpers.client.subscriptions.update(*fixture['url_params'])
    body = fixture['body']['subscriptions']

    assert_is_instance(response, resources.Subscription)
    assert_is_none(responses.calls[-1].request.headers.get('Idempotency-Key'))
    assert_equal(response.amount, body.get('amount'))
    assert_equal(response.app_fee, body.get('app_fee'))
    assert_equal(response.count, body.get('count'))
    assert_equal(response.created_at, body.get('created_at'))
    assert_equal(response.currency, body.get('currency'))
    assert_equal(response.day_of_month, body.get('day_of_month'))
    assert_equal(response.earliest_charge_date_after_resume, body.get('earliest_charge_date_after_resume'))
    assert_equal(response.end_date, body.get('end_date'))
    assert_equal(response.id, body.get('id'))
    assert_equal(response.interval, body.get('interval'))
    assert_equal(response.interval_unit, body.get('interval_unit'))
    assert_equal(response.metadata, body.get('metadata'))
    assert_equal(response.month, body.get('month'))
    assert_equal(response.name, body.get('name'))
    assert_equal(response.payment_reference, body.get('payment_reference'))
    assert_equal(response.retry_if_possible, body.get('retry_if_possible'))
    assert_equal(response.start_date, body.get('start_date'))
    assert_equal(response.status, body.get('status'))
    assert_equal(response.upcoming_payments, body.get('upcoming_payments'))
    assert_equal(response.links.mandate,
                 body.get('links')['mandate'])

@responses.activate
def test_timeout_subscriptions_update_retries():
    fixture = helpers.load_fixture('subscriptions')['update']
    with helpers.stub_timeout_then_response(fixture) as rsps:
      response = helpers.client.subscriptions.update(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['subscriptions']

    assert_is_instance(response, resources.Subscription)

def test_502_subscriptions_update_retries():
    fixture = helpers.load_fixture('subscriptions')['update']
    with helpers.stub_502_then_response(fixture) as rsps:
      response = helpers.client.subscriptions.update(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['subscriptions']

    assert_is_instance(response, resources.Subscription)
  

@responses.activate
def test_subscriptions_pause():
    fixture = helpers.load_fixture('subscriptions')['pause']
    helpers.stub_response(fixture)
    response = helpers.client.subscriptions.pause(*fixture['url_params'])
    body = fixture['body']['subscriptions']

    assert_is_instance(response, resources.Subscription)
    assert_is_not_none(responses.calls[-1].request.headers.get('Idempotency-Key'))
    assert_equal(response.amount, body.get('amount'))
    assert_equal(response.app_fee, body.get('app_fee'))
    assert_equal(response.count, body.get('count'))
    assert_equal(response.created_at, body.get('created_at'))
    assert_equal(response.currency, body.get('currency'))
    assert_equal(response.day_of_month, body.get('day_of_month'))
    assert_equal(response.earliest_charge_date_after_resume, body.get('earliest_charge_date_after_resume'))
    assert_equal(response.end_date, body.get('end_date'))
    assert_equal(response.id, body.get('id'))
    assert_equal(response.interval, body.get('interval'))
    assert_equal(response.interval_unit, body.get('interval_unit'))
    assert_equal(response.metadata, body.get('metadata'))
    assert_equal(response.month, body.get('month'))
    assert_equal(response.name, body.get('name'))
    assert_equal(response.payment_reference, body.get('payment_reference'))
    assert_equal(response.retry_if_possible, body.get('retry_if_possible'))
    assert_equal(response.start_date, body.get('start_date'))
    assert_equal(response.status, body.get('status'))
    assert_equal(response.upcoming_payments, body.get('upcoming_payments'))
    assert_equal(response.links.mandate,
                 body.get('links')['mandate'])

def test_timeout_subscriptions_pause_doesnt_retry():
    fixture = helpers.load_fixture('subscriptions')['pause']
    with helpers.stub_timeout(fixture) as rsps:
      with assert_raises(requests.ConnectTimeout):
        response = helpers.client.subscriptions.pause(*fixture['url_params'])
      assert_equal(1, len(rsps.calls))

def test_502_subscriptions_pause_doesnt_retry():
    fixture = helpers.load_fixture('subscriptions')['pause']
    with helpers.stub_502(fixture) as rsps:
      with assert_raises(MalformedResponseError):
        response = helpers.client.subscriptions.pause(*fixture['url_params'])
      assert_equal(1, len(rsps.calls))
  

@responses.activate
def test_subscriptions_resume():
    fixture = helpers.load_fixture('subscriptions')['resume']
    helpers.stub_response(fixture)
    response = helpers.client.subscriptions.resume(*fixture['url_params'])
    body = fixture['body']['subscriptions']

    assert_is_instance(response, resources.Subscription)
    assert_is_not_none(responses.calls[-1].request.headers.get('Idempotency-Key'))
    assert_equal(response.amount, body.get('amount'))
    assert_equal(response.app_fee, body.get('app_fee'))
    assert_equal(response.count, body.get('count'))
    assert_equal(response.created_at, body.get('created_at'))
    assert_equal(response.currency, body.get('currency'))
    assert_equal(response.day_of_month, body.get('day_of_month'))
    assert_equal(response.earliest_charge_date_after_resume, body.get('earliest_charge_date_after_resume'))
    assert_equal(response.end_date, body.get('end_date'))
    assert_equal(response.id, body.get('id'))
    assert_equal(response.interval, body.get('interval'))
    assert_equal(response.interval_unit, body.get('interval_unit'))
    assert_equal(response.metadata, body.get('metadata'))
    assert_equal(response.month, body.get('month'))
    assert_equal(response.name, body.get('name'))
    assert_equal(response.payment_reference, body.get('payment_reference'))
    assert_equal(response.retry_if_possible, body.get('retry_if_possible'))
    assert_equal(response.start_date, body.get('start_date'))
    assert_equal(response.status, body.get('status'))
    assert_equal(response.upcoming_payments, body.get('upcoming_payments'))
    assert_equal(response.links.mandate,
                 body.get('links')['mandate'])

def test_timeout_subscriptions_resume_doesnt_retry():
    fixture = helpers.load_fixture('subscriptions')['resume']
    with helpers.stub_timeout(fixture) as rsps:
      with assert_raises(requests.ConnectTimeout):
        response = helpers.client.subscriptions.resume(*fixture['url_params'])
      assert_equal(1, len(rsps.calls))

def test_502_subscriptions_resume_doesnt_retry():
    fixture = helpers.load_fixture('subscriptions')['resume']
    with helpers.stub_502(fixture) as rsps:
      with assert_raises(MalformedResponseError):
        response = helpers.client.subscriptions.resume(*fixture['url_params'])
      assert_equal(1, len(rsps.calls))
  

@responses.activate
def test_subscriptions_cancel():
    fixture = helpers.load_fixture('subscriptions')['cancel']
    helpers.stub_response(fixture)
    response = helpers.client.subscriptions.cancel(*fixture['url_params'])
    body = fixture['body']['subscriptions']

    assert_is_instance(response, resources.Subscription)
    assert_is_not_none(responses.calls[-1].request.headers.get('Idempotency-Key'))
    assert_equal(response.amount, body.get('amount'))
    assert_equal(response.app_fee, body.get('app_fee'))
    assert_equal(response.count, body.get('count'))
    assert_equal(response.created_at, body.get('created_at'))
    assert_equal(response.currency, body.get('currency'))
    assert_equal(response.day_of_month, body.get('day_of_month'))
    assert_equal(response.earliest_charge_date_after_resume, body.get('earliest_charge_date_after_resume'))
    assert_equal(response.end_date, body.get('end_date'))
    assert_equal(response.id, body.get('id'))
    assert_equal(response.interval, body.get('interval'))
    assert_equal(response.interval_unit, body.get('interval_unit'))
    assert_equal(response.metadata, body.get('metadata'))
    assert_equal(response.month, body.get('month'))
    assert_equal(response.name, body.get('name'))
    assert_equal(response.payment_reference, body.get('payment_reference'))
    assert_equal(response.retry_if_possible, body.get('retry_if_possible'))
    assert_equal(response.start_date, body.get('start_date'))
    assert_equal(response.status, body.get('status'))
    assert_equal(response.upcoming_payments, body.get('upcoming_payments'))
    assert_equal(response.links.mandate,
                 body.get('links')['mandate'])

def test_timeout_subscriptions_cancel_doesnt_retry():
    fixture = helpers.load_fixture('subscriptions')['cancel']
    with helpers.stub_timeout(fixture) as rsps:
      with assert_raises(requests.ConnectTimeout):
        response = helpers.client.subscriptions.cancel(*fixture['url_params'])
      assert_equal(1, len(rsps.calls))

def test_502_subscriptions_cancel_doesnt_retry():
    fixture = helpers.load_fixture('subscriptions')['cancel']
    with helpers.stub_502(fixture) as rsps:
      with assert_raises(MalformedResponseError):
        response = helpers.client.subscriptions.cancel(*fixture['url_params'])
      assert_equal(1, len(rsps.calls))
  



# WARNING: Do not edit by hand, this file was generated by Crank:
#
#   https://github.com/gocardless/crank
#

class PayoutItem(object):
    """A thin wrapper around a payout_item, providing easy access to its
    attributes.

    Example:
      payout_item = client.payout_items.get()
      payout_item.id
    """

    def __init__(self, attributes, api_response):
        self.attributes = attributes
        self.api_response = api_response

    @property
    def amount(self):
        return self.attributes.get('amount')
  

    @property
    def links(self):
        return self.Links(self.attributes.get('links'))
  

    @property
    def taxes(self):
        return self.attributes.get('taxes')
  

    @property
    def type(self):
        return self.attributes.get('type')
  


  

  
    class Links(object):
        """Wrapper for the response's 'links' attribute."""

        def __init__(self, attributes):
            self.attributes = attributes
    
        @property
        def mandate(self):
            return self.attributes.get('mandate')
    
        @property
        def payment(self):
            return self.attributes.get('payment')
    
        @property
        def refund(self):
            return self.attributes.get('refund')
    
  

  

  




# WARNING: Do not edit by hand, this file was generated by Crank:
#
#   https://github.com/gocardless/crank
#

import json

import requests
import responses
from nose.tools import (
  assert_equal,
  assert_is_instance,
  assert_is_none,
  assert_is_not_none,
  assert_not_equal,
  assert_raises
)

from gocardless_pro.errors import MalformedResponseError
from gocardless_pro import resources
from gocardless_pro import list_response

from .. import helpers
  

@responses.activate
def test_mandate_import_entries_create():
    fixture = helpers.load_fixture('mandate_import_entries')['create']
    helpers.stub_response(fixture)
    response = helpers.client.mandate_import_entries.create(*fixture['url_params'])
    body = fixture['body']['mandate_import_entries']

    assert_is_instance(response, resources.MandateImportEntry)
    assert_is_not_none(responses.calls[-1].request.headers.get('Idempotency-Key'))
    assert_equal(response.created_at, body.get('created_at'))
    assert_equal(response.record_identifier, body.get('record_identifier'))
    assert_equal(response.links.customer,
                 body.get('links')['customer'])
    assert_equal(response.links.customer_bank_account,
                 body.get('links')['customer_bank_account'])
    assert_equal(response.links.mandate,
                 body.get('links')['mandate'])
    assert_equal(response.links.mandate_import,
                 body.get('links')['mandate_import'])

@responses.activate
def test_timeout_mandate_import_entries_create_retries():
    fixture = helpers.load_fixture('mandate_import_entries')['create']
    with helpers.stub_timeout_then_response(fixture) as rsps:
      response = helpers.client.mandate_import_entries.create(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['mandate_import_entries']

    assert_is_instance(response, resources.MandateImportEntry)

def test_502_mandate_import_entries_create_retries():
    fixture = helpers.load_fixture('mandate_import_entries')['create']
    with helpers.stub_502_then_response(fixture) as rsps:
      response = helpers.client.mandate_import_entries.create(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['mandate_import_entries']

    assert_is_instance(response, resources.MandateImportEntry)
  

@responses.activate
def test_mandate_import_entries_list():
    fixture = helpers.load_fixture('mandate_import_entries')['list']
    helpers.stub_response(fixture)
    response = helpers.client.mandate_import_entries.list(*fixture['url_params'])
    body = fixture['body']['mandate_import_entries']

    assert_is_instance(response, list_response.ListResponse)
    assert_is_instance(response.records[0], resources.MandateImportEntry)

    assert_equal(response.before, fixture['body']['meta']['cursors']['before'])
    assert_equal(response.after, fixture['body']['meta']['cursors']['after'])
    assert_is_none(responses.calls[-1].request.headers.get('Idempotency-Key'))
    assert_equal([r.created_at for r in response.records],
                 [b.get('created_at') for b in body])
    assert_equal([r.record_identifier for r in response.records],
                 [b.get('record_identifier') for b in body])

@responses.activate
def test_timeout_mandate_import_entries_list_retries():
    fixture = helpers.load_fixture('mandate_import_entries')['list']
    with helpers.stub_timeout_then_response(fixture) as rsps:
      response = helpers.client.mandate_import_entries.list(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['mandate_import_entries']

    assert_is_instance(response, list_response.ListResponse)
    assert_is_instance(response.records[0], resources.MandateImportEntry)

    assert_equal(response.before, fixture['body']['meta']['cursors']['before'])
    assert_equal(response.after, fixture['body']['meta']['cursors']['after'])

def test_502_mandate_import_entries_list_retries():
    fixture = helpers.load_fixture('mandate_import_entries')['list']
    with helpers.stub_502_then_response(fixture) as rsps:
      response = helpers.client.mandate_import_entries.list(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['mandate_import_entries']

    assert_is_instance(response, list_response.ListResponse)
    assert_is_instance(response.records[0], resources.MandateImportEntry)

    assert_equal(response.before, fixture['body']['meta']['cursors']['before'])
    assert_equal(response.after, fixture['body']['meta']['cursors']['after'])

@responses.activate
def test_mandate_import_entries_all():
    fixture = helpers.load_fixture('mandate_import_entries')['list']

    def callback(request):
        if 'after=123' in request.url:
          fixture['body']['meta']['cursors']['after'] = None
        else:
          fixture['body']['meta']['cursors']['after'] = '123'
        return [200, {}, json.dumps(fixture['body'])]

    url = 'http://example.com' + fixture['path_template']
    responses.add_callback(fixture['method'], url, callback)

    all_records = list(helpers.client.mandate_import_entries.all())
    assert_equal(len(all_records), len(fixture['body']['mandate_import_entries']) * 2)
    for record in all_records:
      assert_is_instance(record, resources.MandateImportEntry)
    
  



# WARNING: Do not edit by hand, this file was generated by Crank:
#
#   https://github.com/gocardless/crank
#

import json

import requests
import responses
from nose.tools import (
  assert_equal,
  assert_is_instance,
  assert_is_none,
  assert_is_not_none,
  assert_not_equal,
  assert_raises
)

from gocardless_pro.errors import MalformedResponseError
from gocardless_pro import resources
from gocardless_pro import list_response

from .. import helpers
  

@responses.activate
def test_billing_request_templates_list():
    fixture = helpers.load_fixture('billing_request_templates')['list']
    helpers.stub_response(fixture)
    response = helpers.client.billing_request_templates.list(*fixture['url_params'])
    body = fixture['body']['billing_request_templates']

    assert_is_instance(response, list_response.ListResponse)
    assert_is_instance(response.records[0], resources.BillingRequestTemplate)

    assert_equal(response.before, fixture['body']['meta']['cursors']['before'])
    assert_equal(response.after, fixture['body']['meta']['cursors']['after'])
    assert_is_none(responses.calls[-1].request.headers.get('Idempotency-Key'))
    assert_equal([r.authorisation_url for r in response.records],
                 [b.get('authorisation_url') for b in body])
    assert_equal([r.created_at for r in response.records],
                 [b.get('created_at') for b in body])
    assert_equal([r.id for r in response.records],
                 [b.get('id') for b in body])
    assert_equal([r.mandate_request_currency for r in response.records],
                 [b.get('mandate_request_currency') for b in body])
    assert_equal([r.mandate_request_metadata for r in response.records],
                 [b.get('mandate_request_metadata') for b in body])
    assert_equal([r.mandate_request_scheme for r in response.records],
                 [b.get('mandate_request_scheme') for b in body])
    assert_equal([r.mandate_request_verify for r in response.records],
                 [b.get('mandate_request_verify') for b in body])
    assert_equal([r.metadata for r in response.records],
                 [b.get('metadata') for b in body])
    assert_equal([r.name for r in response.records],
                 [b.get('name') for b in body])
    assert_equal([r.payment_request_amount for r in response.records],
                 [b.get('payment_request_amount') for b in body])
    assert_equal([r.payment_request_currency for r in response.records],
                 [b.get('payment_request_currency') for b in body])
    assert_equal([r.payment_request_description for r in response.records],
                 [b.get('payment_request_description') for b in body])
    assert_equal([r.payment_request_metadata for r in response.records],
                 [b.get('payment_request_metadata') for b in body])
    assert_equal([r.payment_request_scheme for r in response.records],
                 [b.get('payment_request_scheme') for b in body])
    assert_equal([r.redirect_uri for r in response.records],
                 [b.get('redirect_uri') for b in body])
    assert_equal([r.updated_at for r in response.records],
                 [b.get('updated_at') for b in body])

@responses.activate
def test_timeout_billing_request_templates_list_retries():
    fixture = helpers.load_fixture('billing_request_templates')['list']
    with helpers.stub_timeout_then_response(fixture) as rsps:
      response = helpers.client.billing_request_templates.list(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['billing_request_templates']

    assert_is_instance(response, list_response.ListResponse)
    assert_is_instance(response.records[0], resources.BillingRequestTemplate)

    assert_equal(response.before, fixture['body']['meta']['cursors']['before'])
    assert_equal(response.after, fixture['body']['meta']['cursors']['after'])

def test_502_billing_request_templates_list_retries():
    fixture = helpers.load_fixture('billing_request_templates')['list']
    with helpers.stub_502_then_response(fixture) as rsps:
      response = helpers.client.billing_request_templates.list(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['billing_request_templates']

    assert_is_instance(response, list_response.ListResponse)
    assert_is_instance(response.records[0], resources.BillingRequestTemplate)

    assert_equal(response.before, fixture['body']['meta']['cursors']['before'])
    assert_equal(response.after, fixture['body']['meta']['cursors']['after'])

@responses.activate
def test_billing_request_templates_all():
    fixture = helpers.load_fixture('billing_request_templates')['list']

    def callback(request):
        if 'after=123' in request.url:
          fixture['body']['meta']['cursors']['after'] = None
        else:
          fixture['body']['meta']['cursors']['after'] = '123'
        return [200, {}, json.dumps(fixture['body'])]

    url = 'http://example.com' + fixture['path_template']
    responses.add_callback(fixture['method'], url, callback)

    all_records = list(helpers.client.billing_request_templates.all())
    assert_equal(len(all_records), len(fixture['body']['billing_request_templates']) * 2)
    for record in all_records:
      assert_is_instance(record, resources.BillingRequestTemplate)
    
  

@responses.activate
def test_billing_request_templates_get():
    fixture = helpers.load_fixture('billing_request_templates')['get']
    helpers.stub_response(fixture)
    response = helpers.client.billing_request_templates.get(*fixture['url_params'])
    body = fixture['body']['billing_request_templates']

    assert_is_instance(response, resources.BillingRequestTemplate)
    assert_is_none(responses.calls[-1].request.headers.get('Idempotency-Key'))
    assert_equal(response.authorisation_url, body.get('authorisation_url'))
    assert_equal(response.created_at, body.get('created_at'))
    assert_equal(response.id, body.get('id'))
    assert_equal(response.mandate_request_currency, body.get('mandate_request_currency'))
    assert_equal(response.mandate_request_metadata, body.get('mandate_request_metadata'))
    assert_equal(response.mandate_request_scheme, body.get('mandate_request_scheme'))
    assert_equal(response.mandate_request_verify, body.get('mandate_request_verify'))
    assert_equal(response.metadata, body.get('metadata'))
    assert_equal(response.name, body.get('name'))
    assert_equal(response.payment_request_amount, body.get('payment_request_amount'))
    assert_equal(response.payment_request_currency, body.get('payment_request_currency'))
    assert_equal(response.payment_request_description, body.get('payment_request_description'))
    assert_equal(response.payment_request_metadata, body.get('payment_request_metadata'))
    assert_equal(response.payment_request_scheme, body.get('payment_request_scheme'))
    assert_equal(response.redirect_uri, body.get('redirect_uri'))
    assert_equal(response.updated_at, body.get('updated_at'))

@responses.activate
def test_timeout_billing_request_templates_get_retries():
    fixture = helpers.load_fixture('billing_request_templates')['get']
    with helpers.stub_timeout_then_response(fixture) as rsps:
      response = helpers.client.billing_request_templates.get(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['billing_request_templates']

    assert_is_instance(response, resources.BillingRequestTemplate)

def test_502_billing_request_templates_get_retries():
    fixture = helpers.load_fixture('billing_request_templates')['get']
    with helpers.stub_502_then_response(fixture) as rsps:
      response = helpers.client.billing_request_templates.get(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['billing_request_templates']

    assert_is_instance(response, resources.BillingRequestTemplate)
  

@responses.activate
def test_billing_request_templates_create():
    fixture = helpers.load_fixture('billing_request_templates')['create']
    helpers.stub_response(fixture)
    response = helpers.client.billing_request_templates.create(*fixture['url_params'])
    body = fixture['body']['billing_request_templates']

    assert_is_instance(response, resources.BillingRequestTemplate)
    assert_is_not_none(responses.calls[-1].request.headers.get('Idempotency-Key'))
    assert_equal(response.authorisation_url, body.get('authorisation_url'))
    assert_equal(response.created_at, body.get('created_at'))
    assert_equal(response.id, body.get('id'))
    assert_equal(response.mandate_request_currency, body.get('mandate_request_currency'))
    assert_equal(response.mandate_request_metadata, body.get('mandate_request_metadata'))
    assert_equal(response.mandate_request_scheme, body.get('mandate_request_scheme'))
    assert_equal(response.mandate_request_verify, body.get('mandate_request_verify'))
    assert_equal(response.metadata, body.get('metadata'))
    assert_equal(response.name, body.get('name'))
    assert_equal(response.payment_request_amount, body.get('payment_request_amount'))
    assert_equal(response.payment_request_currency, body.get('payment_request_currency'))
    assert_equal(response.payment_request_description, body.get('payment_request_description'))
    assert_equal(response.payment_request_metadata, body.get('payment_request_metadata'))
    assert_equal(response.payment_request_scheme, body.get('payment_request_scheme'))
    assert_equal(response.redirect_uri, body.get('redirect_uri'))
    assert_equal(response.updated_at, body.get('updated_at'))

@responses.activate
def test_billing_request_templates_create_new_idempotency_key_for_each_call():
    fixture = helpers.load_fixture('billing_request_templates')['create']
    helpers.stub_response(fixture)
    helpers.client.billing_request_templates.create(*fixture['url_params'])
    helpers.client.billing_request_templates.create(*fixture['url_params'])
    assert_not_equal(responses.calls[0].request.headers.get('Idempotency-Key'),
                     responses.calls[1].request.headers.get('Idempotency-Key'))

def test_timeout_billing_request_templates_create_idempotency_conflict():
    create_fixture = helpers.load_fixture('billing_request_templates')['create']
    get_fixture = helpers.load_fixture('billing_request_templates')['get']
    with helpers.stub_timeout_then_idempotency_conflict(create_fixture, get_fixture) as rsps:
      response = helpers.client.billing_request_templates.create(*create_fixture['url_params'])
      assert_equal(2, len(rsps.calls))

    assert_is_instance(response, resources.BillingRequestTemplate)

@responses.activate
def test_timeout_billing_request_templates_create_retries():
    fixture = helpers.load_fixture('billing_request_templates')['create']
    with helpers.stub_timeout_then_response(fixture) as rsps:
      response = helpers.client.billing_request_templates.create(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['billing_request_templates']

    assert_is_instance(response, resources.BillingRequestTemplate)

def test_502_billing_request_templates_create_retries():
    fixture = helpers.load_fixture('billing_request_templates')['create']
    with helpers.stub_502_then_response(fixture) as rsps:
      response = helpers.client.billing_request_templates.create(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['billing_request_templates']

    assert_is_instance(response, resources.BillingRequestTemplate)
  

@responses.activate
def test_billing_request_templates_update():
    fixture = helpers.load_fixture('billing_request_templates')['update']
    helpers.stub_response(fixture)
    response = helpers.client.billing_request_templates.update(*fixture['url_params'])
    body = fixture['body']['billing_request_templates']

    assert_is_instance(response, resources.BillingRequestTemplate)
    assert_is_none(responses.calls[-1].request.headers.get('Idempotency-Key'))
    assert_equal(response.authorisation_url, body.get('authorisation_url'))
    assert_equal(response.created_at, body.get('created_at'))
    assert_equal(response.id, body.get('id'))
    assert_equal(response.mandate_request_currency, body.get('mandate_request_currency'))
    assert_equal(response.mandate_request_metadata, body.get('mandate_request_metadata'))
    assert_equal(response.mandate_request_scheme, body.get('mandate_request_scheme'))
    assert_equal(response.mandate_request_verify, body.get('mandate_request_verify'))
    assert_equal(response.metadata, body.get('metadata'))
    assert_equal(response.name, body.get('name'))
    assert_equal(response.payment_request_amount, body.get('payment_request_amount'))
    assert_equal(response.payment_request_currency, body.get('payment_request_currency'))
    assert_equal(response.payment_request_description, body.get('payment_request_description'))
    assert_equal(response.payment_request_metadata, body.get('payment_request_metadata'))
    assert_equal(response.payment_request_scheme, body.get('payment_request_scheme'))
    assert_equal(response.redirect_uri, body.get('redirect_uri'))
    assert_equal(response.updated_at, body.get('updated_at'))

@responses.activate
def test_timeout_billing_request_templates_update_retries():
    fixture = helpers.load_fixture('billing_request_templates')['update']
    with helpers.stub_timeout_then_response(fixture) as rsps:
      response = helpers.client.billing_request_templates.update(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['billing_request_templates']

    assert_is_instance(response, resources.BillingRequestTemplate)

def test_502_billing_request_templates_update_retries():
    fixture = helpers.load_fixture('billing_request_templates')['update']
    with helpers.stub_502_then_response(fixture) as rsps:
      response = helpers.client.billing_request_templates.update(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['billing_request_templates']

    assert_is_instance(response, resources.BillingRequestTemplate)
  



import codecs
from setuptools import setup, find_packages

with codecs.open('README.rst', 'r', 'utf-8') as file:
    long_description = file.read()

setup(
    name = 'gocardless_pro',
    version = '1.28.0',
    packages = find_packages(exclude=['tests']),
    install_requires = ['requests>=2.6', 'six'],
    author = 'GoCardless',
    author_email = 'engineering@gocardless.com',
    description = 'A client library for the GoCardless Pro API.',
    long_description = long_description,
    license = 'MIT',
    keywords = 'gocardless directdebit payments sepa bacs',
    url = 'https://github.com/gocardless/gocardless-pro-python',
)



# WARNING: Do not edit by hand, this file was generated by Crank:
#
#   https://github.com/gocardless/crank
#

import json

import requests
import responses
from nose.tools import (
  assert_equal,
  assert_is_instance,
  assert_is_none,
  assert_is_not_none,
  assert_not_equal,
  assert_raises
)

from gocardless_pro.errors import MalformedResponseError
from gocardless_pro import resources
from gocardless_pro import list_response

from .. import helpers
  

@responses.activate
def test_customer_notifications_handle():
    fixture = helpers.load_fixture('customer_notifications')['handle']
    helpers.stub_response(fixture)
    response = helpers.client.customer_notifications.handle(*fixture['url_params'])
    body = fixture['body']['customer_notifications']

    assert_is_instance(response, resources.CustomerNotification)
    assert_is_not_none(responses.calls[-1].request.headers.get('Idempotency-Key'))
    assert_equal(response.action_taken, body.get('action_taken'))
    assert_equal(response.action_taken_at, body.get('action_taken_at'))
    assert_equal(response.action_taken_by, body.get('action_taken_by'))
    assert_equal(response.id, body.get('id'))
    assert_equal(response.type, body.get('type'))
    assert_equal(response.links.customer,
                 body.get('links')['customer'])
    assert_equal(response.links.event,
                 body.get('links')['event'])
    assert_equal(response.links.mandate,
                 body.get('links')['mandate'])
    assert_equal(response.links.payment,
                 body.get('links')['payment'])
    assert_equal(response.links.refund,
                 body.get('links')['refund'])
    assert_equal(response.links.subscription,
                 body.get('links')['subscription'])

def test_timeout_customer_notifications_handle_doesnt_retry():
    fixture = helpers.load_fixture('customer_notifications')['handle']
    with helpers.stub_timeout(fixture) as rsps:
      with assert_raises(requests.ConnectTimeout):
        response = helpers.client.customer_notifications.handle(*fixture['url_params'])
      assert_equal(1, len(rsps.calls))

def test_502_customer_notifications_handle_doesnt_retry():
    fixture = helpers.load_fixture('customer_notifications')['handle']
    with helpers.stub_502(fixture) as rsps:
      with assert_raises(MalformedResponseError):
        response = helpers.client.customer_notifications.handle(*fixture['url_params'])
      assert_equal(1, len(rsps.calls))
  



# WARNING: Do not edit by hand, this file was generated by Crank:
#
#   https://github.com/gocardless/crank
#

from . import base_service
from .. import resources
from ..paginator import Paginator
from .. import errors

class WebhooksService(base_service.BaseService):
    """Service class that provides access to the webhooks
    endpoints of the GoCardless Pro API.
    """

    RESOURCE_CLASS = resources.Webhook
    RESOURCE_NAME = 'webhooks'


    def list(self,params=None, headers=None):
        """List webhooks.

        Returns a [cursor-paginated](#api-usage-cursor-pagination) list of your
        webhooks.

        Args:
              params (dict, optional): Query string parameters.

        Returns:
              ListResponse of Webhook instances
        """
        path = '/webhooks'
        

        response = self._perform_request('GET', path, params, headers,
                                         retry_failures=True)
        return self._resource_for(response)

    def all(self, params=None):
        if params is None:
            params = {}
        return Paginator(self, params)
    
  

    def get(self,identity,params=None, headers=None):
        """Get a single webhook.

        Retrieves the details of an existing webhook.

        Args:
              identity (string): Unique identifier, beginning with "WB".
              params (dict, optional): Query string parameters.

        Returns:
              Webhook
        """
        path = self._sub_url_params('/webhooks/:identity', {
          
            'identity': identity,
          })
        

        response = self._perform_request('GET', path, params, headers,
                                         retry_failures=True)
        return self._resource_for(response)
  

    def retry(self,identity,params=None, headers=None):
        """Retry a webhook.

        Requests for a previous webhook to be sent again

        Args:
              identity (string): Unique identifier, beginning with "WB".
              params (dict, optional): Request body.

        Returns:
              Webhook
        """
        path = self._sub_url_params('/webhooks/:identity/actions/retry', {
          
            'identity': identity,
          })
        
        if params is not None:
            params = {'data': params}
        response = self._perform_request('POST', path, params, headers,
                                         retry_failures=False)
        return self._resource_for(response)
  



# WARNING: Do not edit by hand, this file was generated by Crank:
#
#   https://github.com/gocardless/crank
#

class PayerAuthorisation(object):
    """A thin wrapper around a payer_authorisation, providing easy access to its
    attributes.

    Example:
      payer_authorisation = client.payer_authorisations.get()
      payer_authorisation.id
    """

    def __init__(self, attributes, api_response):
        self.attributes = attributes
        self.api_response = api_response

    @property
    def bank_account(self):
        return self.BankAccount(self.attributes.get('bank_account'))
  

    @property
    def created_at(self):
        return self.attributes.get('created_at')
  

    @property
    def customer(self):
        return self.Customer(self.attributes.get('customer'))
  

    @property
    def id(self):
        return self.attributes.get('id')
  

    @property
    def incomplete_fields(self):
        return self.attributes.get('incomplete_fields')
  

    @property
    def links(self):
        return self.Links(self.attributes.get('links'))
  

    @property
    def mandate(self):
        return self.Mandate(self.attributes.get('mandate'))
  

    @property
    def status(self):
        return self.attributes.get('status')
  


  
    class BankAccount(object):
        """Wrapper for the response's 'bank_account' attribute."""

        def __init__(self, attributes):
            self.attributes = attributes
    
        @property
        def account_holder_name(self):
            return self.attributes.get('account_holder_name')
    
        @property
        def account_number(self):
            return self.attributes.get('account_number')
    
        @property
        def account_number_ending(self):
            return self.attributes.get('account_number_ending')
    
        @property
        def account_number_suffix(self):
            return self.attributes.get('account_number_suffix')
    
        @property
        def account_type(self):
            return self.attributes.get('account_type')
    
        @property
        def bank_code(self):
            return self.attributes.get('bank_code')
    
        @property
        def branch_code(self):
            return self.attributes.get('branch_code')
    
        @property
        def country_code(self):
            return self.attributes.get('country_code')
    
        @property
        def currency(self):
            return self.attributes.get('currency')
    
        @property
        def iban(self):
            return self.attributes.get('iban')
    
        @property
        def metadata(self):
            return self.attributes.get('metadata')
    
  

  

  
    class Customer(object):
        """Wrapper for the response's 'customer' attribute."""

        def __init__(self, attributes):
            self.attributes = attributes
    
        @property
        def address_line1(self):
            return self.attributes.get('address_line1')
    
        @property
        def address_line2(self):
            return self.attributes.get('address_line2')
    
        @property
        def address_line3(self):
            return self.attributes.get('address_line3')
    
        @property
        def city(self):
            return self.attributes.get('city')
    
        @property
        def company_name(self):
            return self.attributes.get('company_name')
    
        @property
        def country_code(self):
            return self.attributes.get('country_code')
    
        @property
        def danish_identity_number(self):
            return self.attributes.get('danish_identity_number')
    
        @property
        def email(self):
            return self.attributes.get('email')
    
        @property
        def family_name(self):
            return self.attributes.get('family_name')
    
        @property
        def given_name(self):
            return self.attributes.get('given_name')
    
        @property
        def locale(self):
            return self.attributes.get('locale')
    
        @property
        def metadata(self):
            return self.attributes.get('metadata')
    
        @property
        def postal_code(self):
            return self.attributes.get('postal_code')
    
        @property
        def region(self):
            return self.attributes.get('region')
    
        @property
        def swedish_identity_number(self):
            return self.attributes.get('swedish_identity_number')
    
  

  

  

  
    class Links(object):
        """Wrapper for the response's 'links' attribute."""

        def __init__(self, attributes):
            self.attributes = attributes
    
        @property
        def bank_account(self):
            return self.attributes.get('bank_account')
    
        @property
        def customer(self):
            return self.attributes.get('customer')
    
        @property
        def mandate(self):
            return self.attributes.get('mandate')
    
  

  
    class Mandate(object):
        """Wrapper for the response's 'mandate' attribute."""

        def __init__(self, attributes):
            self.attributes = attributes
    
        @property
        def metadata(self):
            return self.attributes.get('metadata')
    
        @property
        def payer_ip_address(self):
            return self.attributes.get('payer_ip_address')
    
        @property
        def reference(self):
            return self.attributes.get('reference')
    
        @property
        def scheme(self):
            return self.attributes.get('scheme')
    
  

  




# WARNING: Do not edit by hand, this file was generated by Crank:
#
#   https://github.com/gocardless/crank
#

import json

import requests
import responses
from nose.tools import (
  assert_equal,
  assert_is_instance,
  assert_is_none,
  assert_is_not_none,
  assert_not_equal,
  assert_raises
)

from gocardless_pro.errors import MalformedResponseError
from gocardless_pro import resources
from gocardless_pro import list_response

from .. import helpers
  

@responses.activate
def test_events_list():
    fixture = helpers.load_fixture('events')['list']
    helpers.stub_response(fixture)
    response = helpers.client.events.list(*fixture['url_params'])
    body = fixture['body']['events']

    assert_is_instance(response, list_response.ListResponse)
    assert_is_instance(response.records[0], resources.Event)

    assert_equal(response.before, fixture['body']['meta']['cursors']['before'])
    assert_equal(response.after, fixture['body']['meta']['cursors']['after'])
    assert_is_none(responses.calls[-1].request.headers.get('Idempotency-Key'))
    assert_equal([r.action for r in response.records],
                 [b.get('action') for b in body])
    assert_equal([r.created_at for r in response.records],
                 [b.get('created_at') for b in body])
    assert_equal([r.customer_notifications for r in response.records],
                 [b.get('customer_notifications') for b in body])
    assert_equal([r.id for r in response.records],
                 [b.get('id') for b in body])
    assert_equal([r.metadata for r in response.records],
                 [b.get('metadata') for b in body])
    assert_equal([r.resource_type for r in response.records],
                 [b.get('resource_type') for b in body])

@responses.activate
def test_timeout_events_list_retries():
    fixture = helpers.load_fixture('events')['list']
    with helpers.stub_timeout_then_response(fixture) as rsps:
      response = helpers.client.events.list(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['events']

    assert_is_instance(response, list_response.ListResponse)
    assert_is_instance(response.records[0], resources.Event)

    assert_equal(response.before, fixture['body']['meta']['cursors']['before'])
    assert_equal(response.after, fixture['body']['meta']['cursors']['after'])

def test_502_events_list_retries():
    fixture = helpers.load_fixture('events')['list']
    with helpers.stub_502_then_response(fixture) as rsps:
      response = helpers.client.events.list(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['events']

    assert_is_instance(response, list_response.ListResponse)
    assert_is_instance(response.records[0], resources.Event)

    assert_equal(response.before, fixture['body']['meta']['cursors']['before'])
    assert_equal(response.after, fixture['body']['meta']['cursors']['after'])

@responses.activate
def test_events_all():
    fixture = helpers.load_fixture('events')['list']

    def callback(request):
        if 'after=123' in request.url:
          fixture['body']['meta']['cursors']['after'] = None
        else:
          fixture['body']['meta']['cursors']['after'] = '123'
        return [200, {}, json.dumps(fixture['body'])]

    url = 'http://example.com' + fixture['path_template']
    responses.add_callback(fixture['method'], url, callback)

    all_records = list(helpers.client.events.all())
    assert_equal(len(all_records), len(fixture['body']['events']) * 2)
    for record in all_records:
      assert_is_instance(record, resources.Event)
    
  

@responses.activate
def test_events_get():
    fixture = helpers.load_fixture('events')['get']
    helpers.stub_response(fixture)
    response = helpers.client.events.get(*fixture['url_params'])
    body = fixture['body']['events']

    assert_is_instance(response, resources.Event)
    assert_is_none(responses.calls[-1].request.headers.get('Idempotency-Key'))
    assert_equal(response.action, body.get('action'))
    assert_equal(response.created_at, body.get('created_at'))
    assert_equal(response.customer_notifications, body.get('customer_notifications'))
    assert_equal(response.id, body.get('id'))
    assert_equal(response.metadata, body.get('metadata'))
    assert_equal(response.resource_type, body.get('resource_type'))
    assert_equal(response.details.bank_account_id,
                 body.get('details')['bank_account_id'])
    assert_equal(response.details.cause,
                 body.get('details')['cause'])
    assert_equal(response.details.currency,
                 body.get('details')['currency'])
    assert_equal(response.details.description,
                 body.get('details')['description'])
    assert_equal(response.details.not_retried_reason,
                 body.get('details')['not_retried_reason'])
    assert_equal(response.details.origin,
                 body.get('details')['origin'])
    assert_equal(response.details._property,
                 body.get('details')['property'])
    assert_equal(response.details.reason_code,
                 body.get('details')['reason_code'])
    assert_equal(response.details.scheme,
                 body.get('details')['scheme'])
    assert_equal(response.details.will_attempt_retry,
                 body.get('details')['will_attempt_retry'])
    assert_equal(response.links.bank_authorisation,
                 body.get('links')['bank_authorisation'])
    assert_equal(response.links.billing_request,
                 body.get('links')['billing_request'])
    assert_equal(response.links.billing_request_flow,
                 body.get('links')['billing_request_flow'])
    assert_equal(response.links.creditor,
                 body.get('links')['creditor'])
    assert_equal(response.links.customer,
                 body.get('links')['customer'])
    assert_equal(response.links.customer_bank_account,
                 body.get('links')['customer_bank_account'])
    assert_equal(response.links.instalment_schedule,
                 body.get('links')['instalment_schedule'])
    assert_equal(response.links.mandate,
                 body.get('links')['mandate'])
    assert_equal(response.links.mandate_request_mandate,
                 body.get('links')['mandate_request_mandate'])
    assert_equal(response.links.new_customer_bank_account,
                 body.get('links')['new_customer_bank_account'])
    assert_equal(response.links.new_mandate,
                 body.get('links')['new_mandate'])
    assert_equal(response.links.organisation,
                 body.get('links')['organisation'])
    assert_equal(response.links.parent_event,
                 body.get('links')['parent_event'])
    assert_equal(response.links.payer_authorisation,
                 body.get('links')['payer_authorisation'])
    assert_equal(response.links.payment,
                 body.get('links')['payment'])
    assert_equal(response.links.payment_request_payment,
                 body.get('links')['payment_request_payment'])
    assert_equal(response.links.payout,
                 body.get('links')['payout'])
    assert_equal(response.links.previous_customer_bank_account,
                 body.get('links')['previous_customer_bank_account'])
    assert_equal(response.links.refund,
                 body.get('links')['refund'])
    assert_equal(response.links.subscription,
                 body.get('links')['subscription'])

@responses.activate
def test_timeout_events_get_retries():
    fixture = helpers.load_fixture('events')['get']
    with helpers.stub_timeout_then_response(fixture) as rsps:
      response = helpers.client.events.get(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['events']

    assert_is_instance(response, resources.Event)

def test_502_events_get_retries():
    fixture = helpers.load_fixture('events')['get']
    with helpers.stub_502_then_response(fixture) as rsps:
      response = helpers.client.events.get(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['events']

    assert_is_instance(response, resources.Event)
  



# WARNING: Do not edit by hand, this file was generated by Crank:
#
#   https://github.com/gocardless/crank
#

import json

import requests
import responses
from nose.tools import (
  assert_equal,
  assert_is_instance,
  assert_is_none,
  assert_is_not_none,
  assert_not_equal,
  assert_raises
)

from gocardless_pro.errors import MalformedResponseError
from gocardless_pro import resources
from gocardless_pro import list_response

from .. import helpers
  

@responses.activate
def test_mandate_pdfs_create():
    fixture = helpers.load_fixture('mandate_pdfs')['create']
    helpers.stub_response(fixture)
    response = helpers.client.mandate_pdfs.create(*fixture['url_params'])
    body = fixture['body']['mandate_pdfs']

    assert_is_instance(response, resources.MandatePdf)
    assert_is_not_none(responses.calls[-1].request.headers.get('Idempotency-Key'))
    assert_equal(response.expires_at, body.get('expires_at'))
    assert_equal(response.url, body.get('url'))

@responses.activate
def test_timeout_mandate_pdfs_create_retries():
    fixture = helpers.load_fixture('mandate_pdfs')['create']
    with helpers.stub_timeout_then_response(fixture) as rsps:
      response = helpers.client.mandate_pdfs.create(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['mandate_pdfs']

    assert_is_instance(response, resources.MandatePdf)

def test_502_mandate_pdfs_create_retries():
    fixture = helpers.load_fixture('mandate_pdfs')['create']
    with helpers.stub_502_then_response(fixture) as rsps:
      response = helpers.client.mandate_pdfs.create(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['mandate_pdfs']

    assert_is_instance(response, resources.MandatePdf)
  



# WARNING: Do not edit by hand, this file was generated by Crank:
#
#   https://github.com/gocardless/crank
#

from . import base_service
from .. import resources
from ..paginator import Paginator
from .. import errors

class PaymentsService(base_service.BaseService):
    """Service class that provides access to the payments
    endpoints of the GoCardless Pro API.
    """

    RESOURCE_CLASS = resources.Payment
    RESOURCE_NAME = 'payments'


    def create(self,params=None, headers=None):
        """Create a payment.

        <a name="mandate_is_inactive"></a>Creates a new payment object.
        
        This fails with a `mandate_is_inactive` error if the linked
        [mandate](#core-endpoints-mandates) is cancelled or has failed.
        Payments can be created against mandates with status of:
        `pending_customer_approval`, `pending_submission`, `submitted`, and
        `active`.

        Args:
              params (dict, optional): Request body.

        Returns:
              Payment
        """
        path = '/payments'
        
        if params is not None:
            params = {self._envelope_key(): params}

        try:
          response = self._perform_request('POST', path, params, headers,
                                            retry_failures=True)
        except errors.IdempotentCreationConflictError as err:
          if self.raise_on_idempotency_conflict:
            raise err
          return self.get(identity=err.conflicting_resource_id,
                          params=params,
                          headers=headers)
        return self._resource_for(response)
  

    def list(self,params=None, headers=None):
        """List payments.

        Returns a [cursor-paginated](#api-usage-cursor-pagination) list of your
        payments.

        Args:
              params (dict, optional): Query string parameters.

        Returns:
              ListResponse of Payment instances
        """
        path = '/payments'
        

        response = self._perform_request('GET', path, params, headers,
                                         retry_failures=True)
        return self._resource_for(response)

    def all(self, params=None):
        if params is None:
            params = {}
        return Paginator(self, params)
    
  

    def get(self,identity,params=None, headers=None):
        """Get a single payment.

        Retrieves the details of a single existing payment.

        Args:
              identity (string): Unique identifier, beginning with "PM".
              params (dict, optional): Query string parameters.

        Returns:
              Payment
        """
        path = self._sub_url_params('/payments/:identity', {
          
            'identity': identity,
          })
        

        response = self._perform_request('GET', path, params, headers,
                                         retry_failures=True)
        return self._resource_for(response)
  

    def update(self,identity,params=None, headers=None):
        """Update a payment.

        Updates a payment object. This accepts only the metadata parameter.

        Args:
              identity (string): Unique identifier, beginning with "PM".
              params (dict, optional): Request body.

        Returns:
              Payment
        """
        path = self._sub_url_params('/payments/:identity', {
          
            'identity': identity,
          })
        
        if params is not None:
            params = {self._envelope_key(): params}

        response = self._perform_request('PUT', path, params, headers,
                                         retry_failures=True)
        return self._resource_for(response)
  

    def cancel(self,identity,params=None, headers=None):
        """Cancel a payment.

        Cancels the payment if it has not already been submitted to the banks.
        Any metadata supplied to this endpoint will be stored on the payment
        cancellation event it causes.
        
        This will fail with a `cancellation_failed` error unless the payment's
        status is `pending_submission`.

        Args:
              identity (string): Unique identifier, beginning with "PM".
              params (dict, optional): Request body.

        Returns:
              Payment
        """
        path = self._sub_url_params('/payments/:identity/actions/cancel', {
          
            'identity': identity,
          })
        
        if params is not None:
            params = {'data': params}
        response = self._perform_request('POST', path, params, headers,
                                         retry_failures=False)
        return self._resource_for(response)
  

    def retry(self,identity,params=None, headers=None):
        """Retry a payment.

        <a name="retry_failed"></a>Retries a failed payment if the underlying
        mandate is active. You will receive a `resubmission_requested` webhook,
        but after that retrying the payment follows the same process as its
        initial creation, so you will receive a `submitted` webhook, followed
        by a `confirmed` or `failed` event. Any metadata supplied to this
        endpoint will be stored against the payment submission event it causes.
        
        This will return a `retry_failed` error if the payment has not failed.
        
        Payments can be retried up to 3 times.

        Args:
              identity (string): Unique identifier, beginning with "PM".
              params (dict, optional): Request body.

        Returns:
              Payment
        """
        path = self._sub_url_params('/payments/:identity/actions/retry', {
          
            'identity': identity,
          })
        
        if params is not None:
            params = {'data': params}
        response = self._perform_request('POST', path, params, headers,
                                         retry_failures=False)
        return self._resource_for(response)
  



# WARNING: Do not edit by hand, this file was generated by Crank:
#
#   https://github.com/gocardless/crank
#

import json

import requests
import responses
from nose.tools import (
  assert_equal,
  assert_is_instance,
  assert_is_none,
  assert_is_not_none,
  assert_not_equal,
  assert_raises
)

from gocardless_pro.errors import MalformedResponseError
from gocardless_pro import resources
from gocardless_pro import list_response

from .. import helpers
  

@responses.activate
def test_billing_request_flows_create():
    fixture = helpers.load_fixture('billing_request_flows')['create']
    helpers.stub_response(fixture)
    response = helpers.client.billing_request_flows.create(*fixture['url_params'])
    body = fixture['body']['billing_request_flows']

    assert_is_instance(response, resources.BillingRequestFlow)
    assert_is_not_none(responses.calls[-1].request.headers.get('Idempotency-Key'))
    assert_equal(response.authorisation_url, body.get('authorisation_url'))
    assert_equal(response.auto_fulfil, body.get('auto_fulfil'))
    assert_equal(response.created_at, body.get('created_at'))
    assert_equal(response.exit_uri, body.get('exit_uri'))
    assert_equal(response.expires_at, body.get('expires_at'))
    assert_equal(response.id, body.get('id'))
    assert_equal(response.lock_bank_account, body.get('lock_bank_account'))
    assert_equal(response.lock_customer_details, body.get('lock_customer_details'))
    assert_equal(response.redirect_uri, body.get('redirect_uri'))
    assert_equal(response.session_token, body.get('session_token'))
    assert_equal(response.show_redirect_buttons, body.get('show_redirect_buttons'))
    assert_equal(response.links.billing_request,
                 body.get('links')['billing_request'])

@responses.activate
def test_timeout_billing_request_flows_create_retries():
    fixture = helpers.load_fixture('billing_request_flows')['create']
    with helpers.stub_timeout_then_response(fixture) as rsps:
      response = helpers.client.billing_request_flows.create(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['billing_request_flows']

    assert_is_instance(response, resources.BillingRequestFlow)

def test_502_billing_request_flows_create_retries():
    fixture = helpers.load_fixture('billing_request_flows')['create']
    with helpers.stub_502_then_response(fixture) as rsps:
      response = helpers.client.billing_request_flows.create(*fixture['url_params'])
      assert_equal(2, len(rsps.calls))
      assert_equal(rsps.calls[0].request.headers.get('Idempotency-Key'),
                   rsps.calls[1].request.headers.get('Idempotency-Key'))
    body = fixture['body']['billing_request_flows']

    assert_is_instance(response, resources.BillingRequestFlow)
  

@responses.activate
def test_billing_request_flows_initialise():
    fixture = helpers.load_fixture('billing_request_flows')['initialise']
    helpers.stub_response(fixture)
    response = helpers.client.billing_request_flows.initialise(*fixture['url_params'])
    body = fixture['body']['billing_request_flows']

    assert_is_instance(response, resources.BillingRequestFlow)
    assert_is_not_none(responses.calls[-1].request.headers.get('Idempotency-Key'))
    assert_equal(response.authorisation_url, body.get('authorisation_url'))
    assert_equal(response.auto_fulfil, body.get('auto_fulfil'))
    assert_equal(response.created_at, body.get('created_at'))
    assert_equal(response.exit_uri, body.get('exit_uri'))
    assert_equal(response.expires_at, body.get('expires_at'))
    assert_equal(response.id, body.get('id'))
    assert_equal(response.lock_bank_account, body.get('lock_bank_account'))
    assert_equal(response.lock_customer_details, body.get('lock_customer_details'))
    assert_equal(response.redirect_uri, body.get('redirect_uri'))
    assert_equal(response.session_token, body.get('session_token'))
    assert_equal(response.show_redirect_buttons, body.get('show_redirect_buttons'))
    assert_equal(response.links.billing_request,
                 body.get('links')['billing_request'])

def test_timeout_billing_request_flows_initialise_doesnt_retry():
    fixture = helpers.load_fixture('billing_request_flows')['initialise']
    with helpers.stub_timeout(fixture) as rsps:
      with assert_raises(requests.ConnectTimeout):
        response = helpers.client.billing_request_flows.initialise(*fixture['url_params'])
      assert_equal(1, len(rsps.calls))

def test_502_billing_request_flows_initialise_doesnt_retry():
    fixture = helpers.load_fixture('billing_request_flows')['initialise']
    with helpers.stub_502(fixture) as rsps:
      with assert_raises(MalformedResponseError):
        response = helpers.client.billing_request_flows.initialise(*fixture['url_params'])
      assert_equal(1, len(rsps.calls))
  



# WARNING: Do not edit by hand, this file was generated by Crank:
#
#   https://github.com/gocardless/crank
#

from . import base_service
from .. import resources
from ..paginator import Paginator
from .. import errors

class CustomerNotificationsService(base_service.BaseService):
    """Service class that provides access to the customer_notifications
    endpoints of the GoCardless Pro API.
    """

    RESOURCE_CLASS = resources.CustomerNotification
    RESOURCE_NAME = 'customer_notifications'


    def handle(self,identity,params=None, headers=None):
        """Handle a notification.

        "Handling" a notification means that you have sent the notification
        yourself (and
        don't want GoCardless to send it).
        If the notification has already been actioned, or the deadline to
        notify has passed,
        this endpoint will return an `already_actioned` error and you should
        not take
        further action. This endpoint takes no additional parameters.
        

        Args:
              identity (string): The id of the notification.
              params (dict, optional): Request body.

        Returns:
              CustomerNotification
        """
        path = self._sub_url_params('/customer_notifications/:identity/actions/handle', {
          
            'identity': identity,
          })
        
        if params is not None:
            params = {'data': params}
        response = self._perform_request('POST', path, params, headers,
                                         retry_failures=False)
        return self._resource_for(response)
  
